@@ -290,12 +290,12 @@ class DebusineHttpClient:
                 f'Cannot connect to {url}. Error: {str(exc)}'
             ) from exc
 
-        if response.status_code in expected_statuses:
+        if error := self._debusine_problem(response):
+            raise exceptions.DebusineError(error)
+        elif response.status_code in expected_statuses:
             return self._handle_response_content(response, expected_class, url)
         elif response.status_code == requests.codes.not_found:
             raise exceptions.NotFoundError(f'Not found ({url})')
-        elif error := self._debusine_problem(response):
-            raise exceptions.DebusineError(error)
         elif response.status_code == requests.codes.forbidden:
             raise exceptions.ClientForbiddenError(
                 f"HTTP 403. Token ({self._token}) is invalid or disabled"
