@@ -19,7 +19,7 @@ import requests
 from requests import status_codes
 
 from debusine.client import exceptions
-from debusine.client.utils import requests_put_or_connection_error
+from debusine.client.client_utils import requests_put_or_connection_error
 from debusine.utils import parse_range_header
 
 if TYPE_CHECKING:
