@@ -90,7 +90,11 @@ class DebusineTests(TestCase):
 
         self.token = "token"
         self.logger = mock.create_autospec(spec=logging.Logger)
-        self.debusine = Debusine(self.api_url, self.token, logger=self.logger)
+        # Add a trailing slash to the API URL to ensure that all parts of
+        # the client strip it correctly.
+        self.debusine = Debusine(
+            self.api_url + "/", self.token, logger=self.logger
+        )
 
     @responses.activate
     def test_work_request_iter(self) -> None:
