@@ -215,6 +215,30 @@ class DebusineHttpClientTests(TestCase):
         ):
             debusine_client._api_request("GET", self.api_url, TestResponse)
 
+    @responses.activate
+    def test_api_request_raise_debusine_error_404(self) -> None:
+        """Raise DebusineError even with response.code == 404."""
+        title = "Error"
+        detail = "Workflow template not found"
+
+        self.responses_add_problem(
+            responses.POST,
+            self.api_url,
+            title,
+            detail=detail,
+            status=requests.codes.not_found,
+        )
+        debusine_client = DebusineHttpClient(self.api_url, self.token)
+        with self.assertRaises(exceptions.DebusineError) as debusine_error:
+            debusine_client._api_request("POST", self.api_url, TestResponse)
+
+        exception = debusine_error.exception
+
+        self.assertEqual(
+            exception.problem,
+            {"title": title, "detail": detail, "validation_errors": None},
+        )
+
     @responses.activate
     def test_api_request_raise_403_error_with_detail(self) -> None:
         """Raise TokenDisabledError: server returned HTTP 403 with detail."""
