@@ -9,17 +9,23 @@
 
 """Data models for db users and authentication."""
 
+import abc
 import hashlib
 import re
 import secrets
 from datetime import datetime, timedelta
+from enum import StrEnum
 from typing import (
+    Annotated,
     Any,
     Generic,
+    Literal,
     Optional,
+    Self,
     TYPE_CHECKING,
     TypeAlias,
     TypeVar,
+    Union,
     assert_never,
     cast,
 )
@@ -34,7 +40,12 @@ from django.db.models import CheckConstraint, Q, QuerySet, UniqueConstraint
 from django.urls import reverse
 from django.utils import timezone
 
-from debusine.db.context import context
+try:
+    import pydantic.v1 as pydantic
+except ImportError:
+    import pydantic as pydantic  # type: ignore
+
+from debusine.db.context import ContextConsistencyError, context
 from debusine.db.models import permissions
 from debusine.db.models.permissions import (
     PartialCheckResult,
@@ -46,6 +57,7 @@ from debusine.db.models.permissions import (
 )
 from debusine.db.models.scopes import Scope
 from debusine.server import notifications
+from debusine.utils.typing_utils import copy_signature_from
 
 if TYPE_CHECKING:
     from django_stubs_ext.db.models import TypedModelMeta
@@ -373,6 +385,11 @@ class GroupQuerySet(QuerySet["Group", A], Generic[A]):
         # Otherwise all groups can be displayed
         return self.all()
 
+    @permission_filter
+    def can_display_audit_log(self, user: PermissionUser) -> "GroupQuerySet[A]":
+        """Keep only groups whose audit log can be displayed."""
+        return self.can_display(user)
+
     @permission_filter
     def can_manage(self, user: PermissionUser) -> "GroupQuerySet[A]":
         """Keep only groups that can be managed."""
@@ -475,6 +492,18 @@ class Group(models.Model):
         """Return the scoped group name."""
         return f"{self.scope.name}/{self.name}"
 
+    @copy_signature_from(models.Model.save)
+    def save(self, *args: Any, **kwargs: Any) -> None:
+        """Update audit log on group create/update."""
+        old: Optional["Group"] = None
+        if self.pk:
+            old = Group.objects.get(pk=self.pk)
+        super().save(*args, **kwargs)
+        if old is None:
+            self.add_audit_log(GroupAuditLogCreated.create(self))
+        else:
+            self.add_audit_log(GroupAuditLogUpdated.create_diff(old, self))
+
     def get_absolute_url(self) -> str:
         """Return an absolute URL to this user."""
         # Prevent circular import
@@ -499,6 +528,11 @@ class Group(models.Model):
         # Otherwise any logged in user can display group details
         return True
 
+    @permission_check("{user} cannot display the audit log of {resource}")
+    def can_display_audit_log(self, user: PermissionUser) -> bool:
+        """Check if the group audit log can be displayed."""
+        return self.can_display(user)
+
     @permission_check("{user} cannot manage {resource}")
     def can_manage(self, user: PermissionUser) -> bool:
         """Check if the group can be managed."""
@@ -555,6 +589,9 @@ class Group(models.Model):
             raise ValueError(f"User {user} is not a member of group {self}")
         membership.role = role
         membership.save()
+        self.add_audit_log(
+            GroupAuditLogMemberRoleChanged(user=user.username, role=role)
+        )
         return membership
 
     def add_user(
@@ -566,6 +603,9 @@ class Group(models.Model):
         )
         if not created:
             raise ValueError(f"{user} is already a member of group {self}")
+        self.add_audit_log(
+            GroupAuditLogMemberAdded(user=user.username, role=role)
+        )
         return membership
 
     def remove_user(self, user: User) -> None:
@@ -575,6 +615,20 @@ class Group(models.Model):
         except GroupMembership.DoesNotExist:
             raise ValueError(f"User {user} is not a member of group {self}")
         membership.delete()
+        self.add_audit_log(GroupAuditLogMemberRemoved(user=user.username))
+
+    def add_audit_log(
+        self, changes: "GroupAuditLogEntry"
+    ) -> Optional["GroupAuditLog"]:
+        """Add an audit log entry."""
+        if context.user is None or not context.user.is_authenticated:
+            if context.permission_checks_disabled:
+                return None
+            raise ContextConsistencyError("add_audit_log requires a valid user")
+        changes.actor = context.user.username
+        return GroupAuditLog.objects.create_entry(
+            group=self, actor=context.user, changes=changes
+        )
 
 
 class GroupMembership(models.Model):
@@ -603,3 +657,205 @@ class GroupMembership(models.Model):
                 name="%(app_label)s_%(class)s_unique_group_user",
             ),
         ]
+
+
+class GroupAuditLogEntryTypes(StrEnum):
+    """Possible values for GroupAuditLogEntry types."""
+
+    CREATED = "create"
+    UPDATED = "update"
+    ADD_USER = "add_user"
+    REMOVE_USER = "del_user"
+    SET_USER_ROLE = "set_role"
+
+
+class GroupAuditLogEntryBase(pydantic.BaseModel, abc.ABC):
+    """Base class for all audit log payloads."""
+
+    class Config:
+        """Set up stricter pydantic Config."""
+
+        validate_assignment = True
+        extra = pydantic.Extra.forbid
+
+    #: Username of the user performing the action
+    actor: str = ""
+
+    @abc.abstractmethod
+    def description(self) -> str:
+        """Format a description for this change."""
+
+
+class GroupAuditLogCreated(GroupAuditLogEntryBase):
+    """Group has been created."""
+
+    type: Literal[GroupAuditLogEntryTypes.CREATED] = (
+        GroupAuditLogEntryTypes.CREATED
+    )
+    name: str
+    scope: str
+    ephemeral: bool
+
+    @classmethod
+    def create(cls, group: Group) -> Self:
+        """Create an entry populated with group's fields."""
+        return cls(
+            name=group.name,
+            scope=str(group.scope),
+            ephemeral=group.ephemeral,
+        )
+
+    def description(self) -> str:
+        """Format a description for this change."""
+        unknown = "<unknown>"
+        group_desc = (
+            f"group {self.name or unknown} in scope {self.scope or unknown}"
+        )
+        if self.ephemeral:
+            return f"{self.actor} created ephemeral {group_desc}"
+        else:
+            return f"{self.actor} created {group_desc}"
+
+
+class GroupAuditLogUpdated(GroupAuditLogEntryBase):
+    """Group has been updated."""
+
+    type: Literal[GroupAuditLogEntryTypes.UPDATED] = (
+        GroupAuditLogEntryTypes.UPDATED
+    )
+    name: tuple[str, str] | None = None
+    scope: tuple[str, str] | None = None
+    ephemeral: tuple[bool, bool] | None = None
+
+    @classmethod
+    def create_diff(cls, old: Group, new: Group) -> Self:
+        """
+        Compute the changes between two Group objects.
+
+        :returns: a dict mapping field names to tuples of ``(old, new)`` values
+        """
+        name: tuple[str, str] | None = None
+        scope: tuple[str, str] | None = None
+        ephemeral: tuple[bool, bool] | None = None
+        if old.name != new.name:
+            name = (old.name, new.name)
+        if old.scope != new.scope:
+            scope = (str(old.scope), str(new.scope))
+        if old.ephemeral != new.ephemeral:
+            ephemeral = (old.ephemeral, new.ephemeral)
+        return cls(name=name, scope=scope, ephemeral=ephemeral)
+
+    def description(self) -> str:
+        """Format a description for this change."""
+        changes: list[str] = []
+        for field in "name", "scope", "ephemeral":
+            change = getattr(self, field)
+            if change is None:
+                continue
+            changes.append(f"{field}: {change[0]!r}→{change[1]!r}")
+        if changes:
+            return f"{self.actor} updated {'; '.join(changes)}"
+        else:
+            return f"{self.actor} updated group, no changes detected"
+
+
+class GroupAuditLogMembershipBase(GroupAuditLogEntryBase):
+    """Base for membership changes descriptions."""
+
+    #: Username of the member affected
+    user: str
+
+
+class GroupAuditLogMemberAdded(GroupAuditLogMembershipBase):
+    """Member added to the gruoup."""
+
+    type: Literal[GroupAuditLogEntryTypes.ADD_USER] = (
+        GroupAuditLogEntryTypes.ADD_USER
+    )
+    role: GroupRoles
+
+    def description(self) -> str:
+        """Format a description for this change."""
+        return f"{self.actor} added {self.user} as {self.role}"
+
+
+class GroupAuditLogMemberRoleChanged(GroupAuditLogMembershipBase):
+    """Member role updated."""
+
+    type: Literal[GroupAuditLogEntryTypes.SET_USER_ROLE] = (
+        GroupAuditLogEntryTypes.SET_USER_ROLE
+    )
+    role: GroupRoles
+
+    def description(self) -> str:
+        """Format a description for this change."""
+        return f"{self.actor} set {self.user}'s role as {self.role}"
+
+
+class GroupAuditLogMemberRemoved(GroupAuditLogMembershipBase):
+    """Member removed from the gruoup."""
+
+    type: Literal[GroupAuditLogEntryTypes.REMOVE_USER] = (
+        GroupAuditLogEntryTypes.REMOVE_USER
+    )
+
+    def description(self) -> str:
+        """Format a description for this change."""
+        return f"{self.actor} removed {self.user}"
+
+
+GroupAuditLogEntry: TypeAlias = Annotated[
+    Union[
+        GroupAuditLogCreated,
+        GroupAuditLogUpdated,
+        GroupAuditLogMemberAdded,
+        GroupAuditLogMemberRoleChanged,
+        GroupAuditLogMemberRemoved,
+    ],
+    pydantic.Field(discriminator="type"),
+]
+
+
+class GroupAuditLogManager(models.Manager["GroupAuditLog"]):
+    """Manager for :py:class:`GroupAuditLog`."""
+
+    def create_entry(
+        self,
+        group: Group,
+        actor: User,
+        changes: GroupAuditLogEntry,
+    ) -> "GroupAuditLog":
+        """Create GroupAuditLog entry."""
+        return self.create(group=group, actor=actor, changes=changes.dict())
+
+
+class GroupAuditLog(models.Model):
+    """Audit log entry for group changes."""
+
+    created_at = models.DateTimeField(auto_now_add=True)
+    group = models.ForeignKey(
+        Group, on_delete=models.CASCADE, related_name="audit_log"
+    )
+    actor = models.ForeignKey(User, on_delete=models.SET_NULL, null=True)
+    changes = models.JSONField()
+
+    objects = GroupAuditLogManager()
+
+    @property
+    def changes_model(self) -> GroupAuditLogEntry:
+        """Return the pydantic model for changes."""
+        # mypy complains that GroupAuditLogEntry is a "<typing special form>",
+        # but the code works. I'm adding a mypy override for now, and hopefully
+        # this can get cleaned in pydantic2.
+        # See https://salsa.debian.org/freexian-team/debusine/-/merge_requests/1701#note_597300  # noqa: E501
+        return pydantic.parse_obj_as(
+            GroupAuditLogEntry, self.changes  # type: ignore[arg-type]
+        )
+
+
+class ClientEnroll(models.Model):
+    """Information from a client wanting to enroll."""
+
+    created_at = models.DateTimeField(auto_now_add=True)
+    nonce = models.CharField(max_length=64, unique=True)
+    payload = models.JSONField(default=dict)
