@@ -201,7 +201,7 @@ class Collection(models.Model):
         return CollectionManagerInterface.get_manager_for(self)
 
     def get_absolute_url(self) -> str:
-        """Return the canonical URL to display the item."""
+        """Return the canonical URL to display the collection."""
         return reverse(
             "workspaces:collections:detail",
             kwargs={
@@ -211,6 +211,17 @@ class Collection(models.Model):
             },
         )
 
+    def get_absolute_url_search(self) -> str:
+        """Return the canonical URL to search inside the collection."""
+        return reverse(
+            "workspaces:collections:search",
+            kwargs={
+                "wname": self.workspace.name,
+                "ccat": self.category,
+                "cname": self.name,
+            },
+        )
+
     @permission_check("{user} cannot display collection {resource}")
     def can_display(self, user: PermissionUser) -> bool:
         """Check if the collection can be displayed."""
