@@ -25,7 +25,7 @@ from typing import (
 import pgtrigger
 from django.core.exceptions import ValidationError
 from django.db import models
-from django.db.models import F, Q, QuerySet, UniqueConstraint
+from django.db.models import CheckConstraint, F, Q, QuerySet, UniqueConstraint
 
 from debusine.db.context import context
 from debusine.db.models import permissions
@@ -232,19 +232,41 @@ class Scope(models.Model):
         # QuerySet sees a CharField, but we know it's a ScopeRoles enum
         return cast(QuerySet["ScopeRole", "ScopeRoles"], result)
 
-    def upload_file_stores(self, fileobj: File) -> QuerySet[FileStore]:
+    def upload_file_stores(
+        self,
+        fileobj: File,
+        *,
+        enforce_soft_limits: bool = False,
+        include_write_only: bool = False,
+    ) -> QuerySet[FileStore]:
         """
         Find the file stores in this scope where `fileobj` can be uploaded.
 
         The returned query set is in descending order of upload priority.
+
+        :param enforce_soft_limits: Enforce `soft_max_size` policies as well
+          as `max_size`.
+        :param include_write_only: Uploading a file to a write-only store
+          will mean that debusine won't download it from there again, so
+          that only makes sense in specialized situations such as populating
+          a backup store.  Set this to True to include write-only stores in
+          the returned query set.
         """
-        return (
+        file_stores = (
             self.file_stores.exclude(filestoreinscope__read_only=True)
             .exclude(max_size__lt=F("total_size") + fileobj.size)
             .order_by(
                 F("filestoreinscope__upload_priority").desc(nulls_last=True)
             )
         )
+        if enforce_soft_limits:
+            # TODO: We should also enforce FileStoreInScope.soft_max_size.
+            file_stores = file_stores.exclude(
+                soft_max_size__lt=F("total_size") + fileobj.size
+            )
+        if not include_write_only:
+            file_stores = file_stores.exclude(filestoreinscope__write_only=True)
+        return file_stores
 
     def upload_file_backend(self, fileobj: File) -> "FileBackendInterface[Any]":
         """
@@ -404,6 +426,12 @@ class FileStoreInScope(models.Model):
                 ),
                 condition=Q(_file_store_instance_wide=False),
             ),
+            # It does not make sense to request a store to be populated
+            # while also requesting it to be either drained or read-only.
+            CheckConstraint(
+                name="%(app_label)s_%(class)s_consistent_populate",
+                check=Q(populate=False) | Q(drain=False, read_only=False),
+            ),
         ]
 
     def __str__(self) -> str:
