@@ -145,7 +145,7 @@ class PermissionsTests(TestCase):
         self.assertQuerySetEqual(
             Scope.objects.filter(ROLES(self.user, Scope.Roles.OWNER)), []
         )
-        self.group.users.add(self.user)
+        self.playground.add_user(self.group, self.user)
         self.assertQuerySetEqual(
             Scope.objects.filter(ROLES(self.user, Scope.Roles.OWNER)),
             [self.scope],
@@ -160,7 +160,7 @@ class PermissionsTests(TestCase):
             ),
             [],
         )
-        self.group.users.add(self.user)
+        self.playground.add_user(self.group, self.user)
         self.assertQuerySetEqual(
             Workspace.objects.filter(
                 ROLES(self.user, Scope.Roles.OWNER, member="scope")
@@ -312,7 +312,7 @@ class PermissionsTests(TestCase):
             artifact, _ = self.playground.create_artifact(create_files=False)
 
         self.assertIsNone(get_resource_scope(self.user))
-        self.assertIsNone(get_resource_scope(self.group))
+        self.assertEqual(get_resource_scope(self.group), self.scope)
 
         self.assertEqual(get_resource_scope(self.scope), self.scope)
         self.assertEqual(get_resource_scope(self.scope1), self.scope1)
