@@ -142,7 +142,7 @@ class ScopeTests(TestCase):
         user = self.scenario.user
         self.assertQuerySetEqual(scope.get_roles(user), [])
 
-        group.users.add(user)
+        self.playground.add_user(group, user)
         self.assertEqual(list(scope.get_roles(user)), [ScopeRole.Roles.OWNER])
 
     def test_get_roles_multiple_groups(self) -> None:
@@ -158,10 +158,10 @@ class ScopeTests(TestCase):
         user = self.scenario.user
         self.assertQuerySetEqual(scope.get_roles(user), [])
 
-        group1.users.add(user)
+        self.playground.add_user(group1, user)
         self.assertEqual(list(scope.get_roles(user)), [ScopeRole.Roles.OWNER])
 
-        group2.users.add(user)
+        self.playground.add_user(group2, user)
         self.assertEqual(list(scope.get_roles(user)), [ScopeRole.Roles.OWNER])
 
     def test_context_has_role_no_context(self) -> None:
@@ -292,6 +292,64 @@ class ScopeTests(TestCase):
             first_store.get_backend_object().db_store,
         )
 
+    def test_upload_file_stores_enforce_soft_limits(self) -> None:
+        """`upload_file_stores` can optionally enforce soft limits."""
+        fileobj = self.playground.create_file(contents=b"test")
+        scope = self.scenario.scope
+        first_store = scope.file_stores.get()
+        soft_max_size_store = FileStore.objects.create(
+            name="near-full",
+            backend=FileStore.BackendChoices.MEMORY,
+            configuration={"name": "near-full"},
+            soft_max_size=5,
+            max_size=10,
+        )
+        scope.file_stores.add(soft_max_size_store)
+
+        # If a store's soft_max_size would be exceeded by adding the new
+        # file, and soft limits are enforced, it is not eligible to accept
+        # the upload.
+        self.playground.create_file_in_backend(
+            soft_max_size_store.get_backend_object(), contents=b"abcd"
+        )
+
+        soft_max_size_store.refresh_from_db()
+        self.assertEqual(soft_max_size_store.total_size, 4)
+        self.assertQuerySetEqual(
+            scope.upload_file_stores(fileobj),
+            [first_store, soft_max_size_store],
+        )
+        self.assertQuerySetEqual(
+            scope.upload_file_stores(fileobj, enforce_soft_limits=True),
+            [first_store],
+        )
+        self.assertEqual(
+            scope.upload_file_backend(fileobj).db_store,
+            first_store.get_backend_object().db_store,
+        )
+
+    def test_upload_file_stores_include_write_only(self) -> None:
+        """`upload_file_stores` can optionally include write-only stores."""
+        fileobj = self.playground.create_file(contents=b"test")
+        scope = self.scenario.scope
+        first_store = scope.file_stores.get()
+        write_only_store = FileStore.objects.create(
+            name="write_only",
+            backend=FileStore.BackendChoices.MEMORY,
+            configuration={"name": "write_only"},
+        )
+        scope.file_stores.add(
+            write_only_store, through_defaults={"write_only": True}
+        )
+
+        self.assertQuerySetEqual(
+            scope.upload_file_stores(fileobj), [first_store]
+        )
+        self.assertQuerySetEqual(
+            scope.upload_file_stores(fileobj, include_write_only=True),
+            [first_store, write_only_store],
+        )
+
     def test_download_file_stores(self) -> None:
         """`download_file_stores` returns stores in the correct order."""
         fileobj = self.playground.create_file()
@@ -442,6 +500,7 @@ class ScopeRoleTests(TestCase):
     group2: ClassVar[Group]
 
     @classmethod
+    @context.disable_permission_checks()
     def setUpTestData(cls) -> None:
         """Set up common test data."""
         super().setUpTestData()
