@@ -15,6 +15,7 @@ from collections import defaultdict
 from collections.abc import Generator
 from contextlib import contextmanager
 from datetime import datetime, timedelta
+from enum import Enum, auto
 from functools import cached_property
 from itertools import chain
 from typing import (
@@ -27,25 +28,29 @@ from typing import (
     TypeAlias,
     TypeVar,
     assert_never,
+    cast,
 )
 
 from django.core.exceptions import FieldError, ValidationError
 from django.core.serializers.json import DjangoJSONEncoder
-from django.db import models
+from django.db import connection, models, transaction
 from django.db.models import (
+    DateTimeField,
     Exists,
     ExpressionWrapper,
     F,
     IntegerField,
     JSONField,
+    Max,
     OuterRef,
     Q,
     QuerySet,
     UniqueConstraint,
 )
-from django.db.models.functions import Coalesce
+from django.db.models.functions import Coalesce, Greatest
 from django.urls import reverse
 from django.utils import timezone
+from django_cte import CTEQuerySet, With
 
 from debusine.artifacts.models import (
     BareDataCategory,
@@ -100,6 +105,9 @@ logger = logging.getLogger(__name__)
 
 A = TypeVar("A")
 
+#: Limit on how many times a work request may be retried automatically.
+MAX_AUTOMATIC_RETRIES = 3
+
 
 class CannotRetry(Exception):
     """Exception raised if a work request cannot be retried."""
@@ -113,6 +121,25 @@ class InternalTaskError(Exception):
     """Exception raised when trying to instantiate an internal task."""
 
 
+class WorkRequestRetryReason(Enum):
+    """The reason for retrying a work request."""
+
+    #: The worker that was running this work request failed or was
+    #: terminated.  Only retry up to the limit set by
+    #: :py:const:`MAX_AUTOMATIC_RETRIES`, just in case this work request is
+    #: killing the worker, and increment the retry count.
+    WORKER_FAILED = auto()
+
+    #: The work request is being automatically retried due to a
+    #: ``retry-with-delays`` action.  Increment the retry count, but ignore
+    #: :py:const:`MAX_AUTOMATIC_RETRIES`; the ``delays`` option sets a limit
+    #: on how many times the work request will be retried.
+    DELAY = auto()
+
+    #: A user requested a retry.  Reset the (automatic) retry count.
+    MANUAL = auto()
+
+
 class WorkflowTemplateQuerySet(QuerySet["WorkflowTemplate", A], Generic[A]):
     """Custom QuerySet for WorkflowTemplate."""
 
@@ -132,6 +159,22 @@ class WorkflowTemplateQuerySet(QuerySet["WorkflowTemplate", A], Generic[A]):
         # Delegate to workspace can_display check
         return self.filter(workspace__in=Workspace.objects.can_display(user))
 
+    @permission_filter
+    def can_run(self, user: PermissionUser) -> "WorkflowTemplateQuerySet[A]":
+        """Keep only WorkflowTemplates that can be started."""
+        assert user is not None  # Enforced by decorator
+        # Workers and anonymous users are not allowed
+        if context.worker_token or not user.is_authenticated:
+            return self.none()
+        return self.filter(
+            WORKSPACE_ROLES(
+                user,
+                scope=[Scope.Roles.OWNER],
+                workspace=[Workspace.Roles.OWNER, Workspace.Roles.CONTRIBUTOR],
+                member="workspace",
+            )
+        ).distinct()
+
 
 class WorkflowTemplateManager(models.Manager["WorkflowTemplate"]):
     """Manager for WorkflowTemplate model."""
@@ -202,6 +245,31 @@ class WorkflowTemplate(models.Model):
         # Delegate to workspace can_display check
         return self.workspace.can_display(user)
 
+    @permission_check("{user} cannot run {resource}")
+    def can_run(self, user: PermissionUser) -> bool:
+        """Check if the workflow template can be run."""
+        assert user is not None  # enforced by decorator
+        # Workers and anonymous users are not allowed
+        if context.worker_token or not user.is_authenticated:
+            return False
+        # Shortcut to avoid hitting the database for common cases
+        match self.workspace.context_has_role(
+            user,
+            (Workspace.Roles.OWNER, Workspace.Roles.CONTRIBUTOR),
+            scope_roles=Scope.Roles.OWNER,
+        ):
+            case PartialCheckResult.ALLOW:
+                return True
+            case PartialCheckResult.DENY:
+                return False
+            case PartialCheckResult.PASS:
+                pass
+            case _ as unreachable:
+                assert_never(unreachable)
+        return (
+            WorkflowTemplate.objects.can_run(user).filter(pk=self.pk).exists()
+        )
+
     def completed_workflows(self) -> int:
         """Count completed workflows for this template in context.workspace."""
         return (
@@ -257,6 +325,39 @@ class WorkRequestQuerySet(QuerySet["WorkRequest", A], Generic[A]):
         """Filter to work requests in the current workspace."""
         return self.filter(workspace=context.require_workspace())
 
+    def can_be_unblocked(self) -> "WorkRequestQuerySet[WorkRequest]":
+        """Filter to work requests that can be unblocked at all."""
+        return cast(
+            WorkRequestQuerySet[WorkRequest],
+            self.filter(
+                # The work request must currently be blocked.
+                Q(status=WorkRequest.Statuses.BLOCKED)
+                # If this work request is in a workflow, then that workflow
+                # must be running.
+                & (
+                    Q(parent__isnull=True)
+                    | Q(parent__status=WorkRequest.Statuses.RUNNING)
+                )
+            ),
+        )
+
+    def can_be_automatically_unblocked(
+        self,
+    ) -> "WorkRequestQuerySet[WorkRequest]":
+        """Filter to work requests that can be automatically unblocked."""
+        return self.can_be_unblocked().filter(
+            # DEPS work requests can be unblocked if and only if all their
+            # dependencies have completed.  We don't know how to deal with
+            # other unblock strategies automatically.
+            Q(unblock_strategy=WorkRequest.UnblockStrategy.DEPS)
+            & ~Exists(
+                WorkRequest.objects.filter(
+                    Q(reverse_dependencies=OuterRef("id"))
+                    & ~Q(status=WorkRequest.Statuses.COMPLETED)
+                )
+            )
+        )
+
     @permission_filter
     def can_display(self, user: PermissionUser) -> "WorkRequestQuerySet[A]":
         """Keep only WorkRequests that can be displayed."""
@@ -335,7 +436,6 @@ class WorkRequestManager(models.Manager["WorkRequest"]):
         *,
         template: WorkflowTemplate,
         data: dict[str, Any],
-        created_by: "User",
         parent: Optional["WorkRequest"] = None,
         status: _WorkRequestStatuses | None = None,
         validate: bool = True,
@@ -345,6 +445,10 @@ class WorkRequestManager(models.Manager["WorkRequest"]):
         from debusine.server.workflows import Workflow
         from debusine.server.workflows.models import WorkRequestWorkflowData
 
+        enforce(template.can_run)
+        created_by = context.require_user()
+        assert created_by.is_authenticated
+
         # Lookup the orchestrator
         workflow_cls = Workflow.from_name(template.task_name)
 
@@ -613,6 +717,10 @@ class WorkRequest(models.Model):
         blank=True,
         null=True,
     )
+    # Flag indicating that debusine.server.celery.update_workflows needs to
+    # update workflow_last_activity_at and workflow_runtime_status on
+    # workflows containing this work request.
+    workflows_need_update = models.BooleanField(default=False)
     result = models.CharField(
         max_length=7,
         choices=Results.choices,
@@ -739,6 +847,11 @@ class WorkRequest(models.Model):
                     )
                 ),
             ),
+            # Handles debusine.server.celery.update_workflows.
+            models.Index(
+                "workflows_need_update",
+                name="%(app_label)s_%(class)s_wf_update_idx",
+            ),
         ]
         permissions = [
             (
@@ -998,7 +1111,7 @@ class WorkRequest(models.Model):
             return False
         return True
 
-    def _retry_workflow(self) -> Self:
+    def _retry_workflow(self, reason: WorkRequestRetryReason) -> Self:
         """Retry logic for workflows."""
         # Import here to prevent circular imports
         from debusine.server.workflows.base import orchestrate_workflow
@@ -1008,9 +1121,15 @@ class WorkRequest(models.Model):
         self.status = self.Statuses.PENDING
         self.mark_running()
 
-        # Increment the retry count
+        # Update the retry count
         workflow_data = self.workflow_data
-        workflow_data.retry_count += 1
+        if reason in {
+            WorkRequestRetryReason.WORKER_FAILED,
+            WorkRequestRetryReason.DELAY,
+        }:
+            workflow_data.retry_count += 1
+        else:
+            workflow_data.retry_count = 0
         self.workflow_data = workflow_data
         self.save()
 
@@ -1027,7 +1146,7 @@ class WorkRequest(models.Model):
                 .order_by("id")
             ):
                 try:
-                    child.retry()
+                    child.retry(reason=reason)
                 except CannotRetry as exc:
                     raise CannotRetry(
                         f"child work request {child.pk} cannot be retried"
@@ -1037,7 +1156,7 @@ class WorkRequest(models.Model):
 
         return self
 
-    def _retry_supersede(self) -> "WorkRequest":
+    def _retry_supersede(self, reason: WorkRequestRetryReason) -> "WorkRequest":
         """Retry logic superseding work requests."""
         work_request = WorkRequest.objects.create(
             workspace=self.workspace,
@@ -1076,9 +1195,15 @@ class WorkRequest(models.Model):
             workflow.completed_at = None
             workflow.save()
 
-        # Increment the retry count
+        # Update the retry count
         workflow_data = work_request.workflow_data
-        workflow_data.retry_count += 1
+        if reason in {
+            WorkRequestRetryReason.WORKER_FAILED,
+            WorkRequestRetryReason.DELAY,
+        }:
+            workflow_data.retry_count += 1
+        else:
+            workflow_data.retry_count = 0
         work_request.workflow_data = workflow_data
         work_request.save()
 
@@ -1100,20 +1225,35 @@ class WorkRequest(models.Model):
                 )
                 debusine_promise.save()
 
-        if self.parent is not None:
-            self._workflow_update_runtime_status(self.parent)
+        self.schedule_workflow_update()
 
         return work_request
 
-    def retry(self) -> "WorkRequest":
-        """Create a WorkRequest that supersedes this one."""
+    def retry(
+        self, reason: WorkRequestRetryReason = WorkRequestRetryReason.MANUAL
+    ) -> "WorkRequest":
+        """
+        Create a WorkRequest that supersedes this one.
+
+        :param reason: The reason for retrying this work request.  If it is
+          ``WORKER_FAILED``, then only allow the retry if the retry count is
+          less than :py:const:`MAX_AUTOMATIC_RETRIES`.  If it is either
+          ``WORKER_FAILED`` or ``DELAY``, then increment the work request's
+          ``retry_count``; otherwise, set it back to 0.
+        """
         enforce(self.can_retry)
         self._verify_retry()
 
+        if (
+            reason == WorkRequestRetryReason.WORKER_FAILED
+            and self.workflow_data.retry_count >= MAX_AUTOMATIC_RETRIES
+        ):
+            raise CannotRetry("Maximum number of automatic retries exceeded")
+
         if self.task_type == TaskTypes.WORKFLOW:
-            return self._retry_workflow()
+            return self._retry_workflow(reason=reason)
         else:
-            return self._retry_supersede()
+            return self._retry_supersede(reason=reason)
 
     def clean(self) -> None:
         """
@@ -1306,9 +1446,7 @@ class WorkRequest(models.Model):
         self.status = self.Statuses.RUNNING
         self.save()
 
-        if self.parent is not None:
-            self._workflow_propagate_last_activity(self.parent, self.started_at)
-            self._workflow_update_runtime_status(self.parent)
+        self.schedule_workflow_update()
 
         logger.debug("Marked WorkRequest %s as running", self.pk)
         return True
@@ -1349,11 +1487,7 @@ class WorkRequest(models.Model):
                 runtime=output_data.runtime_statistics.duration,
             )
 
-        if self.parent is not None:
-            self._workflow_propagate_last_activity(
-                self.parent, self.completed_at
-            )
-            self._workflow_update_runtime_status(self.parent)
+        self.schedule_workflow_update()
 
         logger.debug("Marked WorkRequest %s as completed", self.pk)
         # mark dependencies ready before sending notification
@@ -1592,7 +1726,7 @@ class WorkRequest(models.Model):
                 status=self.Statuses.PENDING,
                 workflow_data=WorkRequestWorkflowData(needs_input=False),
             )
-            wr_retried = self.retry()
+            wr_retried = self.retry(reason=WorkRequestRetryReason.DELAY)
             wr_retried.add_dependency(wr_delay)
 
     def process_record_in_task_history(
@@ -1656,8 +1790,7 @@ class WorkRequest(models.Model):
         self.status = self.Statuses.PENDING
         self.save()
 
-        if self.parent:
-            self._workflow_update_runtime_status(self.parent)
+        self.schedule_workflow_update()
 
         logger.debug("Marked WorkRequest %s as pending", self.pk)
         self.process_event_reactions("on_unblock")
@@ -1722,34 +1855,17 @@ class WorkRequest(models.Model):
 
     def can_be_unblocked(self) -> bool:
         """Return True iff this work request can be unblocked at all."""
-        # The work request must currently be blocked.
-        if self.status != WorkRequest.Statuses.BLOCKED:
-            return False
-
-        # If this work request is in a workflow, then that workflow must
-        # be running.
-        if (
-            self.parent is not None
-            and self.parent.status != WorkRequest.Statuses.RUNNING
-        ):
-            return False
-
-        return True
+        return (
+            WorkRequest.objects.can_be_unblocked().filter(id=self.id).exists()
+        )
 
     def can_be_automatically_unblocked(self) -> bool:
         """Return True iff this work request can be automatically unblocked."""
-        if not self.can_be_unblocked():
-            return False
-
-        if self.unblock_strategy == WorkRequest.UnblockStrategy.DEPS:
-            # This work request can be unblocked if and only if all its
-            # dependencies have completed.
-            return not self.dependencies.filter(
-                ~Q(status=WorkRequest.Statuses.COMPLETED)
-            ).exists()
-        else:
-            # We don't know how to unblock this work request automatically.
-            return False
+        return (
+            WorkRequest.objects.can_be_automatically_unblocked()
+            .filter(id=self.id)
+            .exists()
+        )
 
     def unblock_reverse_dependencies(self) -> None:
         """Unblock reverse dependencies."""
@@ -1762,18 +1878,18 @@ class WorkRequest(models.Model):
             # lookup work requests that depend on the completed work
             # request and unblock them if no other work request is
             # blocking them
-            for rdep in self.reverse_dependencies.filter(status=s.BLOCKED):
-                if rdep.can_be_automatically_unblocked():
-                    rdep.mark_pending()
+            for (
+                rdep
+            ) in self.reverse_dependencies.can_be_automatically_unblocked():
+                rdep.mark_pending()
         else:  # failure and !allow_failure
             for rdep in self.reverse_dependencies.filter(status=s.BLOCKED):
                 rdep.mark_aborted()
 
     def unblock_workflow_children(self) -> None:
         """Unblock children of a workflow that has just started running."""
-        for child in self.children.filter(status=WorkRequest.Statuses.BLOCKED):
-            if child.can_be_automatically_unblocked():
-                child.mark_pending()
+        for child in self.children.can_be_automatically_unblocked():
+            child.mark_pending()
 
     def review_manual_unblock(
         self,
@@ -1857,10 +1973,7 @@ class WorkRequest(models.Model):
         self.unblock_reverse_dependencies()
         if self.parent is not None:
             self.parent.maybe_finish_workflow()
-            self._workflow_propagate_last_activity(
-                self.parent, self.completed_at
-            )
-            self._workflow_update_runtime_status(self.parent)
+        self.schedule_workflow_update()
 
         logger.debug(
             "Marked WorkRequest %s as aborted (from status %s)",
@@ -2037,71 +2150,68 @@ class WorkRequest(models.Model):
         else:
             return template_name
 
-    def _workflow_propagate_last_activity(
-        self, workflow: "WorkRequest", timestamp: datetime
-    ) -> None:
+    def schedule_workflow_update(self) -> None:
         """
-        Propagate the last activity timestamp through the workflow hierarchy.
+        Schedule updates of expensive properties of containing workflows.
 
-        Updates the `workflow_last_activity_at` field of the given workflow and
-        recursively propagates the same timestamp to its parent workflows,
-        if any.
+        The ``workflow_last_activity_at`` and ``workflow_runtime_status``
+        fields of workflows are used in the web UI and need to be kept up to
+        date, but updating them requires scanning the workflow graph and can
+        be expensive.  When making changes to a large workflow graph, much
+        of this work can be amortized: for example, we only need to update
+        the root workflow once even if many work requests under it have
+        changed.  We therefore defer that work until the end of the
+        transaction and run it in a Celery task.
         """
-        assert workflow.task_type == TaskTypes.WORKFLOW
+        from debusine.server.celery import update_workflows
 
-        workflow.workflow_last_activity_at = timestamp
-        workflow.save()
+        if self.parent:
+            self.workflows_need_update = True
+            self.save()
 
-        if workflow.parent is not None:
-            self._workflow_propagate_last_activity(workflow.parent, timestamp)
-
-    @classmethod
-    def _workflow_update_runtime_status(cls, workflow: "WorkRequest") -> None:
-        """Update runtime status of `workflow` and its parent."""
-        assert workflow.task_type == TaskTypes.WORKFLOW
-
-        workflow.workflow_runtime_status = compute_workflow_runtime_status(
-            workflow
-        )
-        workflow.save()
-
-        if workflow.parent:
-            cls._workflow_update_runtime_status(workflow.parent)
-
-    @cached_property
-    def _workflow_flattened(self) -> list["WorkRequest"]:
-        return workflow_flattened(self)
+            # Run the update on commit, but only once per savepoint.  This
+            # uses undocumented Django internals, but if it goes wrong
+            # without an obvious unit test failure, then the worst case
+            # should be some extra workflow updates.
+            if (set(connection.savepoint_ids), update_workflows.delay) not in [
+                hook[:2] for hook in connection.run_on_commit
+            ]:
+                transaction.on_commit(update_workflows.delay)
 
     @cached_property
     def workflow_work_requests_success(self) -> int:
         """Return number of successful work requests."""
-        return sum(
-            child.result == WorkRequest.Results.SUCCESS
-            for child in self._workflow_flattened
+        return (
+            workflow_flattened(self)
+            .filter(result=WorkRequest.Results.SUCCESS)
+            .count()
         )
 
     @cached_property
     def workflow_work_requests_failure(self) -> int:
         """Return number of failed work requests."""
-        return sum(
-            child.result == WorkRequest.Results.FAILURE
-            for child in self._workflow_flattened
+        return (
+            workflow_flattened(self)
+            .filter(result=WorkRequest.Results.FAILURE)
+            .count()
         )
 
     @cached_property
     def workflow_work_requests_pending(self) -> int:
         """Return number of pending work requests."""
-        return sum(
-            child.result == WorkRequest.Statuses.PENDING
-            for child in self._workflow_flattened
+        return (
+            workflow_flattened(self)
+            .filter(status=WorkRequest.Statuses.PENDING)
+            .count()
         )
 
     @cached_property
     def workflow_work_requests_blocked(self) -> int:
         """Return number of blocked work requests."""
-        return sum(
-            child.result == WorkRequest.Statuses.BLOCKED
-            for child in self._workflow_flattened
+        return (
+            workflow_flattened(self)
+            .filter(status=WorkRequest.Statuses.BLOCKED)
+            .count()
         )
 
     @property
@@ -2210,23 +2320,57 @@ class NotificationChannel(models.Model):
         return self.name
 
 
-def workflow_flattened(workflow: WorkRequest) -> list["WorkRequest"]:
+def workflow_flattened(workflow: WorkRequest) -> QuerySet[WorkRequest]:
     """
-    Return list of all the work requests part of the workflow.
+    Return queryset of all the work requests that are part of the workflow.
 
     Does not include work requests of type workflow.
     """
     assert workflow.task_type == TaskTypes.WORKFLOW
 
-    work_requests = []
+    work_request_queryset = CTEQuerySet(
+        type(workflow), using=WorkRequest.objects._db
+    )
 
-    for child in workflow.children.all():
-        if child.task_type == TaskTypes.WORKFLOW:
-            work_requests.extend(workflow_flattened(child))
-        else:
-            work_requests.append(child)
+    def workflow_flattened_cte(cte: With) -> QuerySet[WorkRequest]:
+        recursive = (
+            # .id is evasive action for problems encountered when `workflow`
+            # is from a model retrieved as part of a migration.
+            work_request_queryset.filter(parent=workflow.id)
+            .values(rec_parent=F("parent"), rec_child=F("id"))
+            .union(
+                cte.join(type(workflow), parent=cte.col.rec_child).values(
+                    rec_parent=F("parent"), rec_child=F("id")
+                )
+            )
+        )
+        assert isinstance(recursive, QuerySet)
+        return recursive
 
-    return work_requests
+    cte = With.recursive(workflow_flattened_cte)
+    flattened = (
+        cte.join(work_request_queryset, id=cte.col.rec_child)
+        .with_cte(cte)
+        .exclude(task_type=TaskTypes.WORKFLOW)
+    )
+    assert isinstance(flattened, QuerySet)
+    return flattened
+
+
+def compute_workflow_last_activity(workflow: WorkRequest) -> datetime | None:
+    """
+    Compute the last activity timestamp of a workflow.
+
+    This is the latest ``started_at``/``completed_at`` of its descendants.
+    """
+    last_activity = workflow_flattened(workflow).aggregate(
+        last_activity=Max(
+            Greatest("started_at", "completed_at"),
+            output_field=DateTimeField(blank=True, null=True),
+        )
+    )["last_activity"]
+    assert last_activity is None or isinstance(last_activity, datetime)
+    return last_activity
 
 
 def compute_workflow_runtime_status(
@@ -2242,41 +2386,37 @@ def compute_workflow_runtime_status(
 
     flattened = workflow_flattened(workflow)
 
-    if any(
-        child.task_type == TaskTypes.WAIT
-        and child.status == _WorkRequestStatuses.RUNNING
-        and child.workflow_data_json.get("needs_input")
-        for child in flattened
-    ):
+    if flattened.filter(
+        task_type=TaskTypes.WAIT,
+        status=WorkRequest.Statuses.RUNNING,
+        workflow_data_json__needs_input=True,
+    ).exists():
         return _RuntimeStatuses.NEEDS_INPUT
 
-    if any(
-        child.task_type != TaskTypes.WAIT
-        and child.status == _WorkRequestStatuses.RUNNING
-        for child in flattened
+    if (
+        flattened.exclude(task_type=TaskTypes.WAIT)
+        .filter(status=WorkRequest.Statuses.RUNNING)
+        .exists()
     ):
         return _RuntimeStatuses.RUNNING
 
-    if any(
-        child.task_type == TaskTypes.WAIT
-        and child.status == _WorkRequestStatuses.RUNNING
-        and child.workflow_data_json["needs_input"] is False
-        for child in flattened
-    ):
+    if flattened.filter(
+        task_type=TaskTypes.WAIT,
+        status=WorkRequest.Statuses.RUNNING,
+        workflow_data_json__needs_input=False,
+    ).exists():
         return _RuntimeStatuses.WAITING
 
-    if any(child.status == _WorkRequestStatuses.PENDING for child in flattened):
+    if flattened.filter(status=WorkRequest.Statuses.PENDING).exists():
         return _RuntimeStatuses.PENDING
 
-    if any(child.status == _WorkRequestStatuses.BLOCKED for child in flattened):
+    if flattened.filter(status=WorkRequest.Statuses.BLOCKED).exists():
         return _RuntimeStatuses.BLOCKED
 
-    if any(child.status == _WorkRequestStatuses.ABORTED for child in flattened):
+    if flattened.filter(status=WorkRequest.Statuses.ABORTED).exists():
         return _RuntimeStatuses.ABORTED
 
-    if all(
-        child.status == _WorkRequestStatuses.COMPLETED for child in flattened
-    ):
+    if not flattened.exclude(status=WorkRequest.Statuses.COMPLETED).exists():
         return _RuntimeStatuses.COMPLETED
 
     # Any possibility should have been dealt with
@@ -2285,6 +2425,43 @@ def compute_workflow_runtime_status(
     )
 
 
+def workflow_ancestors(
+    work_requests: QuerySet[WorkRequest],
+) -> QuerySet[WorkRequest]:
+    """
+    Return workflows containing any of these work requests.
+
+    This returns any workflows that are one of the given work requests, or
+    direct or indirect ancestors.
+    """
+    work_request_queryset = CTEQuerySet(
+        WorkRequest, using=WorkRequest.objects._db
+    )
+
+    def workflow_ancestors_cte(cte: With) -> QuerySet[WorkRequest]:
+        recursive = (
+            work_request_queryset.filter(id__in=work_requests)
+            .values(rec_parent=F("parent"), rec_child=F("id"))
+            .union(
+                cte.join(WorkRequest, id=cte.col.rec_parent).values(
+                    rec_parent=F("parent"), rec_child=F("id")
+                )
+            )
+        )
+        assert isinstance(recursive, QuerySet)
+        return recursive
+
+    cte = With.recursive(workflow_ancestors_cte)
+    ancestors = (
+        cte.join(work_request_queryset, id=cte.col.rec_child)
+        .with_cte(cte)
+        .distinct("id")
+        .filter(task_type=TaskTypes.WORKFLOW)
+    )
+    assert isinstance(ancestors, QuerySet)
+    return ancestors
+
+
 class DeleteUnused:
     """
     Delete work requests not currently in use.
