@@ -146,7 +146,11 @@ class WorkerPool(models.Model):
     def terminate_worker(self, worker: "Worker") -> None:
         """Terminate the specified worker instance."""
         # Import here to prevent circular imports
-        from debusine.db.models.work_requests import CannotRetry, WorkRequest
+        from debusine.db.models.work_requests import (
+            CannotRetry,
+            WorkRequest,
+            WorkRequestRetryReason,
+        )
 
         if worker.worker_pool != self:
             raise ValueError(
@@ -180,7 +184,7 @@ class WorkerPool(models.Model):
             running.mark_aborted()
             try:
                 with context.disable_permission_checks():
-                    running.retry()
+                    running.retry(reason=WorkRequestRetryReason.WORKER_FAILED)
             except CannotRetry as e:
                 logger.debug(  # noqa: G200
                     "Cannot retry previously-running work request: %s", e
