@@ -437,7 +437,7 @@ class Playground(ArtifactPlayground):
     @context.disable_permission_checks()
     def create_workflow(
         self,
-        task_name: str = "noop",
+        task_name: str | WorkflowTemplate = "noop",
         task_data: BaseWorkflowData | dict[str, Any] | None = None,
         parent: WorkRequest | None = None,
         status: WorkRequest.Statuses | None = None,
@@ -445,27 +445,40 @@ class Playground(ArtifactPlayground):
         validate: bool = True,
     ) -> WorkRequest:
         """Create a workflow."""
-        workflow_template_name = f"{task_name}-template"
-        try:
-            template = WorkflowTemplate.objects.get(name=workflow_template_name)
-        except WorkflowTemplate.DoesNotExist:
-            template = self.create_workflow_template(
-                name=workflow_template_name, task_name=task_name
-            )
+        if created_by is None:
+            created_by = self.get_default_user()
+
+        if isinstance(task_name, WorkflowTemplate):
+            template = task_name
+        else:
+            workflow_template_name = f"{task_name}-template"
+            try:
+                template = WorkflowTemplate.objects.get(
+                    name=workflow_template_name
+                )
+            except WorkflowTemplate.DoesNotExist:
+                template = self.create_workflow_template(
+                    name=workflow_template_name, task_name=task_name
+                )
 
         if isinstance(task_data, BaseWorkflowData):
             task_data = model_to_json_serializable_dict(
                 task_data, exclude_unset=True
             )
 
-        return WorkRequest.objects.create_workflow(
-            template=template,
-            created_by=created_by or self.get_default_user(),
-            data=task_data or {},
-            parent=parent,
-            status=status,
-            validate=validate,
-        )
+        with context.local():
+            context.reset()
+            context.set_scope(template.workspace.scope)
+            context.set_user(created_by)
+            template.workspace.set_current()
+            with context.disable_permission_checks():
+                return WorkRequest.objects.create_workflow(
+                    template=template,
+                    data=task_data or {},
+                    parent=parent,
+                    status=status,
+                    validate=validate,
+                )
 
     @context.disable_permission_checks()
     def create_work_request(
@@ -483,7 +496,7 @@ class Playground(ArtifactPlayground):
         Return a new instance of WorkRequest.
 
         :param mark_running: if True call mark_running() method
-        :param assign_new_worker: if not None assign worker to the work request
+        :param assign_new_worker: if True assign worker to the work request
         :param result: if not None call mark_completed(result)
         :param expired: True to create an expired work request
           (created_at: 1 year ago, expiration_delay: 1 day)
@@ -550,6 +563,7 @@ class Playground(ArtifactPlayground):
         worker_type: WorkerType = WorkerType.EXTERNAL,
         extra_dynamic_metadata: dict[str, Any] | None = None,
         worker_pool: WorkerPool | None = None,
+        fqdn: str = "computer.lan",
     ) -> Worker:
         """Return a new Worker."""
         if worker_pool is not None:
@@ -560,7 +574,7 @@ class Playground(ArtifactPlayground):
             ).latest("registered_at")
         else:
             worker = Worker.objects.create_with_fqdn(
-                "computer.lan",
+                fqdn,
                 worker_type=worker_type,
                 token=self.create_bare_token(),
             )
