@@ -22,11 +22,14 @@ from debusine.db.models import (
     Token,
     User,
     WorkRequest,
+    Worker,
+    WorkerPool,
     WorkflowTemplate,
     Workspace,
 )
 from debusine.db.models.auth import Identity
 from debusine.server.collections.debian_suite import DebianSuiteManager
+from debusine.tasks.models import WorkerType
 
 if TYPE_CHECKING:
     from debusine.db.playground.playground import Playground
@@ -59,7 +62,8 @@ class Scenario:
         """
         Build the scenario.
 
-        This is run by Playground with permission tests disabled
+        This is run by Playground with permission tests disabled and in a local
+        context that will be restored at the end of the build.
         """
         self.playground = playground
 
@@ -156,10 +160,16 @@ class UIPlayground(DefaultContext):
     source_dpkg: Artifact
     source_udev: Artifact
     template_sbuild: WorkflowTemplate
+    worker_pool: WorkerPool
+    worker_static: Worker
+    worker_celery: Worker
+    worker_in_pool: Worker
+    worker_in_pool_busy: Worker
 
     def build(self, playground: "Playground") -> None:
         """Build the scenario."""
         super().build(playground)
+        self.set_current()
 
         # Set a password for the test user
         self.user.set_password("playground")
@@ -172,12 +182,12 @@ class UIPlayground(DefaultContext):
         Identity.objects.create(
             user=self.user,
             issuer="salsa",
-            subject="playground@debian.org",
+            subject="playground@debian.example.org",
             claims={"test": True},
         )
 
         other_user = User.objects.create_user(
-            username="other", first_name="Other", last_name="User"
+            username="playground-other", first_name="Other", last_name="User"
         )
 
         # Make them owners of the playground workspace...
@@ -188,10 +198,26 @@ class UIPlayground(DefaultContext):
         role_group.set_user_role(self.user, Group.Roles.ADMIN)
 
         # Create a second group with both users in it
-        other_group = Group.objects.create(name="Friends", scope=self.scope)
+        other_group = Group.objects.create(name="Playground", scope=self.scope)
         other_group.add_user(self.user)
         other_group.add_user(other_user)
 
+        # Create a worker pool and some workers
+        self.worker_pool = self.playground.create_worker_pool("playground")
+
+        self.worker_static = self.playground.create_worker(
+            fqdn="playground.lan"
+        )
+        self.worker_celery = self.playground.create_worker(
+            worker_type=WorkerType.CELERY, fqdn="playground.lan"
+        )
+        self.worker_in_pool = self.playground.create_worker(
+            fqdn="playground.lan", worker_pool=self.worker_pool
+        )
+        self.worker_in_pool_busy = self.playground.create_worker(
+            fqdn="playground.lan", worker_pool=self.worker_pool
+        )
+
         # Create a Debian scope, to test multi-scope UI elements
         self.playground.get_or_create_scope(
             "debian", label="Debian", icon="web/icons/debian-openlogo-nd.svg"
@@ -240,15 +266,15 @@ class UIPlayground(DefaultContext):
 
         # Populate the debian:suite collection with artifacts
         wr = self.playground.simulate_package_build(
-            self.source_hello, architecture="amd64"
+            self.source_hello, architecture="amd64", worker=self.worker_static
         )
         self.add_results_to_suite(wr)
         wr = self.playground.simulate_package_build(
-            self.source_dpkg, architecture="amd64"
+            self.source_dpkg, architecture="amd64", worker=self.worker_static
         )
         self.add_results_to_suite(wr)
         wr = self.playground.simulate_package_build(
-            self.source_dpkg, architecture="armhf"
+            self.source_dpkg, architecture="armhf", worker=self.worker_static
         )
         self.add_results_to_suite(wr)
 
@@ -276,6 +302,10 @@ class UIPlayground(DefaultContext):
             task_name="noop",
         )
 
+        wr = self.playground.create_work_request(task_name="noop")
+        wr.assign_worker(self.worker_in_pool_busy)
+        wr.mark_running()
+
         self.simulate_sbuild_workflow(self.template_sbuild, self.source_udev)
 
     def add_results_to_suite(
@@ -329,14 +359,16 @@ class UIPlayground(DefaultContext):
                 "target_distribution": "debian:bookworm",
                 "architectures": ["all", "amd64", "s390x"],
             },
-            created_by=self.user,
         )
         workflow.mark_running()
         workflow.save()
 
         # A successful build
         self.playground.simulate_package_build(
-            source, workflow=workflow, architecture="amd64"
+            source,
+            workflow=workflow,
+            architecture="amd64",
+            worker=self.worker_static,
         )
 
         # A failed build
