@@ -1047,9 +1047,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         """Test simulating a package build in a workflow."""
         playground = Playground()
         template = playground.create_workflow_template("test", "noop")
-        workflow = WorkRequest.objects.create_workflow(
-            template=template, data={}, created_by=playground.get_default_user()
-        )
+        workflow = playground.create_workflow(template, task_data={})
         with context.disable_permission_checks():
             source = playground.create_source_artifact()
             wr = playground.simulate_package_build(source, workflow=workflow)
