@@ -33,7 +33,12 @@ from debusine.db.models import (
 )
 from debusine.db.models.workspaces import DeleteWorkspaces, WorkspaceRole
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
-from debusine.server.management.utils import ResourceRoles, Workspaces
+from debusine.server.management.management_utils import (
+    ResourceRoles,
+    Workspaces,
+    get_scope,
+    get_workspace,
+)
 
 
 class Command(DebusineBaseCommand):
@@ -158,13 +163,6 @@ class Command(DebusineBaseCommand):
             help="Group(s) dropping the role",
         )
 
-    def get_scope(self, scope_name: str) -> Scope:
-        """Lookup a scope by name."""
-        try:
-            return Scope.objects.get(name=scope_name)
-        except Scope.DoesNotExist:
-            raise CommandError(f"Scope {scope_name!r} not found", returncode=3)
-
     def get_scope_and_workspace_name(
         self, scope_workspace: str
     ) -> tuple[Scope, str]:
@@ -176,28 +174,7 @@ class Command(DebusineBaseCommand):
                 returncode=3,
             )
         scope_name, workspace_name = scope_workspace.split("/", 1)
-        return self.get_scope(scope_name), workspace_name
-
-    def get_scope_and_workspace(
-        self, scope_workspace: str
-    ) -> tuple[Scope, Workspace]:
-        """Lookup a scopename/workspacename string."""
-        scope, workspace_name = self.get_scope_and_workspace_name(
-            scope_workspace
-        )
-        return scope, self.get_workspace(scope, workspace_name)
-
-    def get_workspace(self, scope: Scope, workspace_name: str) -> Workspace:
-        """Lookup a workspace in a scope."""
-        try:
-            workspace = Workspace.objects.get(scope=scope, name=workspace_name)
-        except Workspace.DoesNotExist:
-            raise CommandError(
-                f"Workspace {workspace_name!r} not found in scope"
-                f" {scope.name!r}",
-                returncode=3,
-            )
-        return workspace
+        return get_scope(scope_name), workspace_name
 
     def get_group(self, scope: Scope, group_name: str) -> Group:
         """Lookup a group in a scope."""
@@ -291,7 +268,7 @@ class Command(DebusineBaseCommand):
         self, *, scope_workspace: str, new_name: str, **options: Any
     ) -> NoReturn:
         """Rename a workspace."""
-        scope, workspace = self.get_scope_and_workspace(scope_workspace)
+        workspace = get_workspace(scope_workspace)
 
         if new_name == workspace.name:
             raise SystemExit(0)
@@ -314,7 +291,7 @@ class Command(DebusineBaseCommand):
         """List workspaces in a scope."""
         workspaces = Workspace.objects.all()
         if options["scope"] is not None:
-            scope = self.get_scope(options["scope"])
+            scope = get_scope(options["scope"])
             workspaces = workspaces.filter(scope=scope)
         Workspaces(options["yaml"]).print(workspaces, self.stdout)
         raise SystemExit(0)
@@ -323,7 +300,7 @@ class Command(DebusineBaseCommand):
         self, scope_workspace: str, **options: Any
     ) -> NoReturn:
         """List workspace's roles."""
-        scope, workspace = self.get_scope_and_workspace(scope_workspace)
+        workspace = get_workspace(scope_workspace)
 
         workspace_roles = WorkspaceRole.objects.filter(resource=workspace)
         ResourceRoles(options["yaml"]).print(workspace_roles, self.stdout)
@@ -333,10 +310,10 @@ class Command(DebusineBaseCommand):
         self, scope_workspace: str, role: str, groups: list[str], **options: Any
     ) -> NoReturn:
         """Assign workspace's role to groups."""
-        scope, workspace = self.get_scope_and_workspace(scope_workspace)
+        workspace = get_workspace(scope_workspace)
         try:
             for group_name in groups:
-                group = self.get_group(scope, group_name)
+                group = self.get_group(workspace.scope, group_name)
                 group.assign_role(workspace, role)
         except ValueError as exc:
             raise CommandError(
@@ -349,9 +326,9 @@ class Command(DebusineBaseCommand):
         self, scope_workspace: str, role: str, groups: list[str], **options: Any
     ) -> NoReturn:
         """Revoke workspace's role from groups."""
-        scope, workspace = self.get_scope_and_workspace(scope_workspace)
+        workspace = get_workspace(scope_workspace)
         for group_name in groups:
-            group = self.get_group(scope, group_name)
+            group = self.get_group(workspace.scope, group_name)
             WorkspaceRole.objects.filter(
                 resource=workspace, group=group, role=role
             ).delete()
@@ -361,7 +338,7 @@ class Command(DebusineBaseCommand):
         self, *, scope_workspace: str, yes: bool, **options: Any
     ) -> NoReturn:
         """Delete a workspace with associated resources."""
-        scope, workspace = self.get_scope_and_workspace(scope_workspace)
+        workspace = get_workspace(scope_workspace)
 
         # Prevent deletion of default workspace
         if workspace == default_workspace():
