@@ -50,7 +50,7 @@ def terminate_idle(minutes: int = 10) -> bool:
     """
     # List cloud workers that have been idle for more than 10 minutes
     idle_workers = Worker.objects.with_idle_time().filter(
-        idle_time__gt=minutes * 60,  # type: ignore[misc]
+        idle_time__gt=minutes * 60,
         worker_pool__isnull=False,
     )
     has_idle = False
