@@ -18,20 +18,24 @@ from django.db import connections, transaction
 from debusine.db.context import context
 from debusine.db.models import WorkRequest, Worker
 from debusine.server.celery import (
-    CeleryWorkerRequiresServerTask,
+    ServerTaskRunError,
     WorkRequestNotPending,
     run_server_task,
 )
-from debusine.server.tasks import BaseServerTask
 from debusine.server.tasks.models import ServerNoopData
-from debusine.tasks import TaskConfigError
-from debusine.tasks.models import BaseDynamicTaskData, BaseTaskData, TaskTypes
-from debusine.tasks.tests.helper_mixin import TestTaskMixin
+from debusine.server.tasks.tests.helpers import TestBaseServerTask
+from debusine.tasks.models import (
+    BaseDynamicTaskData,
+    BaseTaskData,
+    OutputData,
+    OutputDataError,
+    TaskTypes,
+)
 from debusine.test.django import TestCase, TransactionTestCase
 
 
 class TestTaskNotManagingTransactions(
-    TestTaskMixin, BaseServerTask[BaseTaskData, BaseDynamicTaskData]
+    TestBaseServerTask[BaseTaskData, BaseDynamicTaskData]
 ):
     """A sample server task that does not manage its own transactions."""
 
@@ -42,7 +46,7 @@ class TestTaskNotManagingTransactions(
 
 
 class TestTaskManagingTransactions(
-    TestTaskMixin, BaseServerTask[BaseTaskData, BaseDynamicTaskData]
+    TestBaseServerTask[BaseTaskData, BaseDynamicTaskData]
 ):
     """A sample server task that manages its own transactions."""
 
@@ -99,9 +103,11 @@ class RunServerTaskTests(TestCase):
         )
         work_request.task_name = "nonexistent"
         work_request.save()
+        expected_message = "'nonexistent' is not a registered Server task_name"
         with (
             self.assertLogsContains(
-                "Task: nonexistent does not exist",
+                f"Error running work request Server/nonexistent "
+                f"({work_request.id}): {expected_message}",
                 logger="debusine.server.celery",
                 level=logging.ERROR,
             ),
@@ -109,12 +115,24 @@ class RunServerTaskTests(TestCase):
         ):
             result = run_server_task.apply(args=(work_request.id,))
         self.assertTrue(result.failed())
-        self.assertIsInstance(result.result, ValueError)
+        self.assertIsInstance(result.result, ServerTaskRunError)
+        self.assertEqual(result.result.message, expected_message)
+        self.assertEqual(result.result.code, "setup-failed")
         work_request.refresh_from_db()
         self.assertIsNotNone(work_request.started_at)
         self.assertIsNotNone(work_request.completed_at)
         self.assertEqual(work_request.status, WorkRequest.Statuses.COMPLETED)
         self.assertEqual(work_request.result, WorkRequest.Results.ERROR)
+        self.assertEqual(
+            work_request.output_data,
+            OutputData(
+                errors=[
+                    OutputDataError(
+                        message=expected_message, code="setup-failed"
+                    )
+                ]
+            ),
+        )
 
     def test_bad_task_data(self) -> None:
         """The Celery task fails if the work request has bad task data."""
@@ -123,9 +141,13 @@ class RunServerTaskTests(TestCase):
         )
         work_request.task_data = {"nonexistent": True}
         work_request.save()
+        expected_message = (
+            "Failed to configure: 1 validation error for ServerNoopData"
+        )
         with (
             self.assertLogsContains(
-                "Task: servernoop failed to configure",
+                f"Error running work request Server/servernoop "
+                f"({work_request.id}): {expected_message}",
                 logger="debusine.server.celery",
                 level=logging.ERROR,
             ),
@@ -133,21 +155,37 @@ class RunServerTaskTests(TestCase):
         ):
             result = run_server_task.apply(args=(work_request.id,))
         self.assertTrue(result.failed())
-        self.assertIsInstance(result.result, TaskConfigError)
+        self.assertIsInstance(result.result, ServerTaskRunError)
+        self.assertTrue(result.result.message.startswith(expected_message))
+        self.assertEqual(result.result.code, "configure-failed")
         work_request.refresh_from_db()
         self.assertIsNotNone(work_request.started_at)
         self.assertIsNotNone(work_request.completed_at)
         self.assertEqual(work_request.status, WorkRequest.Statuses.COMPLETED)
         self.assertEqual(work_request.result, WorkRequest.Results.ERROR)
+        assert work_request.output_data is not None
+        self.assertIsInstance(work_request.output_data, OutputData)
+        assert work_request.output_data.errors is not None
+        self.assertEqual(len(work_request.output_data.errors), 1)
+        self.assertTrue(
+            work_request.output_data.errors[0].message.startswith(
+                expected_message
+            )
+        )
+        self.assertEqual(
+            work_request.output_data.errors[0].code, "configure-failed"
+        )
 
     def test_external_task(self) -> None:
         """The Celery task fails if asked to run an external task."""
         work_request = self.create_assigned_work_request(
             task_name="noop", task_data={"result": True}
         )
+        expected_message = "Cannot run on a Celery worker"
         with (
             self.assertLogsContains(
-                "Task: noop cannot run on a Celery worker",
+                f"Error running work request Worker/noop ({work_request.id}): "
+                f"{expected_message}",
                 logger="debusine.server.celery",
                 level=logging.ERROR,
             ),
@@ -155,12 +193,24 @@ class RunServerTaskTests(TestCase):
         ):
             result = run_server_task.apply(args=(work_request.id,))
         self.assertTrue(result.failed())
-        self.assertIsInstance(result.result, CeleryWorkerRequiresServerTask)
+        self.assertIsInstance(result.result, ServerTaskRunError)
+        self.assertEqual(result.result.message, expected_message)
+        self.assertEqual(result.result.code, "wrong-task-type")
         work_request.refresh_from_db()
         self.assertIsNotNone(work_request.started_at)
         self.assertIsNotNone(work_request.completed_at)
         self.assertEqual(work_request.status, WorkRequest.Statuses.COMPLETED)
         self.assertEqual(work_request.result, WorkRequest.Results.ERROR)
+        self.assertEqual(
+            work_request.output_data,
+            OutputData(
+                errors=[
+                    OutputDataError(
+                        message=expected_message, code="wrong-task-type"
+                    )
+                ]
+            ),
+        )
 
     def test_task_raises_exception(self) -> None:
         """The Celery task fails if the task raises an exception."""
@@ -169,9 +219,11 @@ class RunServerTaskTests(TestCase):
             task_name="servernoop",
             task_data=ServerNoopData(exception=True),
         )
+        expected_message = "Execution failed: Client requested an exception"
         with (
             self.assertLogsContains(
-                "Task: servernoop failed to execute",
+                f"Error running work request Server/servernoop "
+                f"({work_request.id}): {expected_message}",
                 logger="debusine.server.celery",
                 level=logging.ERROR,
             ),
@@ -179,13 +231,27 @@ class RunServerTaskTests(TestCase):
         ):
             result = run_server_task.apply(args=(work_request.id,))
         self.assertTrue(result.failed())
-        self.assertIsInstance(result.result, RuntimeError)
-        self.assertEqual(result.result.args, ("Client requested an exception",))
+        self.assertIsInstance(result.result, ServerTaskRunError)
+        self.assertEqual(
+            result.result.message,
+            "Execution failed: Client requested an exception",
+        )
+        self.assertEqual(result.result.code, "execute-failed")
         work_request.refresh_from_db()
         self.assertIsNotNone(work_request.started_at)
         self.assertIsNotNone(work_request.completed_at)
         self.assertEqual(work_request.status, WorkRequest.Statuses.COMPLETED)
         self.assertEqual(work_request.result, WorkRequest.Results.ERROR)
+        self.assertEqual(
+            work_request.output_data,
+            OutputData(
+                errors=[
+                    OutputDataError(
+                        message=expected_message, code="execute-failed"
+                    )
+                ]
+            ),
+        )
 
     def test_task_aborted(self) -> None:
         """If the task is aborted, the Celery task logs but does not fail."""
@@ -212,6 +278,7 @@ class RunServerTaskTests(TestCase):
         self.assertIsNone(work_request.completed_at)
         self.assertEqual(work_request.status, WorkRequest.Statuses.RUNNING)
         self.assertEqual(work_request.result, WorkRequest.Results.NONE)
+        self.assertIsNone(work_request.output_data)
 
     def test_task_returns_false(self) -> None:
         """If the task returns False, the Celery task records that."""
@@ -229,6 +296,7 @@ class RunServerTaskTests(TestCase):
         self.assertIsNotNone(work_request.completed_at)
         self.assertEqual(work_request.status, WorkRequest.Statuses.COMPLETED)
         self.assertEqual(work_request.result, WorkRequest.Results.FAILURE)
+        self.assertIsNone(work_request.output_data)
 
     def test_task_returns_true(self) -> None:
         """If the task returns True, the Celery task records that."""
@@ -246,6 +314,7 @@ class RunServerTaskTests(TestCase):
         self.assertIsNotNone(work_request.completed_at)
         self.assertEqual(work_request.status, WorkRequest.Statuses.COMPLETED)
         self.assertEqual(work_request.result, WorkRequest.Results.SUCCESS)
+        self.assertIsNone(work_request.output_data)
 
 
 class RunServerTaskTransactionTests(TransactionTestCase):
