@@ -50,6 +50,8 @@ from debusine.tasks.models import (
     BaseTaskData,
     LookupMultiple,
     LookupSingle,
+    OutputData,
+    OutputDataError,
     TaskTypes,
 )
 from debusine.tasks.server import TaskDatabaseInterface
@@ -68,10 +70,13 @@ class WorkflowValidationError(Exception):
 class WorkflowRunError(Exception):
     """Running a workflow orchestrator or callback failed."""
 
-    def __init__(self, work_request: WorkRequest, message: str) -> None:
+    def __init__(
+        self, work_request: WorkRequest, message: str, code: str
+    ) -> None:
         """Construct the exception."""
         self.work_request = work_request
         self.message = message
+        self.code = code
 
 
 class ArtifactHasNoArchitecture(Exception):
@@ -136,21 +141,10 @@ class Workflow(BaseServerTask[WD, DTD], metaclass=ABCMeta):
         """
         # Do nothing by default
 
-    def ensure_dynamic_data(self, task_database: TaskDatabaseInterface) -> bool:
+    def ensure_dynamic_data(self, task_database: TaskDatabaseInterface) -> None:
         """Ensure that this workflow's dynamic task data has been computed."""
         if self.work_request.dynamic_task_data is None:
-            try:
-                dynamic_task_data = self.compute_dynamic_data(task_database)
-            except Exception as exc:
-                self.logger.warning(  # noqa: G200
-                    "Workflow orchestrator failed to pre-process task data for "
-                    "WorkRequest %s. Task data: %s Error: %s",
-                    self.work_request.id,
-                    self.work_request.task_data,
-                    exc,
-                )
-                return False
-
+            dynamic_task_data = self.compute_dynamic_data(task_database)
             self.work_request.dynamic_task_data = (
                 model_to_json_serializable_dict(
                     dynamic_task_data, exclude_unset=True
@@ -161,8 +155,6 @@ class Workflow(BaseServerTask[WD, DTD], metaclass=ABCMeta):
                 **self.work_request.dynamic_task_data
             )
 
-        return True
-
     @abstractmethod
     def populate(self) -> None:
         """
@@ -220,6 +212,11 @@ class Workflow(BaseServerTask[WD, DTD], metaclass=ABCMeta):
         Create an event reaction for ``on_success`` to update the collection
         with the relevant artifact.
         """
+        if "/" in name:
+            # This wouldn't work well, because "/" is used to separate
+            # lookup string segments.
+            raise ValueError('Collection item name may not contain "/".')
+
         if data is not None:
             for key, value in data.items():
                 if key.startswith("promise_"):
@@ -233,7 +230,7 @@ class Workflow(BaseServerTask[WD, DTD], metaclass=ABCMeta):
 
         try:
             lookup_single(
-                f"internal@collections/{name}",
+                f"internal@collections/name:{name}",
                 work_request.workspace,
                 user=work_request.created_by or AnonymousUser(),
                 workflow_root=work_request.get_workflow_root(),
@@ -392,38 +389,66 @@ def orchestrate_workflow(work_request: WorkRequest) -> None:
                 try:
                     orchestrator = work_request.get_task()
                 except InternalTaskError as exc:
-                    raise WorkflowRunError(work_request, str(exc)) from exc
+                    raise WorkflowRunError(
+                        work_request, str(exc), "configure-failed"
+                    ) from exc
                 except TaskConfigError as exc:
                     raise WorkflowRunError(
-                        work_request, f"failed to configure: {exc}"
+                        work_request,
+                        f"Failed to configure: {exc}",
+                        "configure-failed",
                     ) from exc
                 else:
                     assert isinstance(orchestrator, Workflow)
 
             case _:
                 raise WorkflowRunError(
-                    work_request, "does not have a workflow orchestrator"
+                    work_request,
+                    "Does not have a workflow orchestrator",
+                    "configure-failed",
                 )
 
-        if not orchestrator.ensure_dynamic_data(
-            TaskDatabase(orchestrator.work_request)
-        ):
-            raise WorkflowRunError(
-                work_request, "failed to compute dynamic data"
-            )
-
         match (work_request.task_type, work_request.task_name):
             case (TaskTypes.INTERNAL, "workflow"):
+                parent = orchestrator.work_request
+                try:
+                    orchestrator.ensure_dynamic_data(TaskDatabase(parent))
+                except Exception as exc:
+                    # Workflow callbacks compute dynamic data for their
+                    # parent workflow.  If this fails, it's probably least
+                    # confusing to log information about both the workflow
+                    # callback and the workflow.
+                    raise WorkflowRunError(
+                        work_request,
+                        f"Failed to compute dynamic data for "
+                        f"{parent.task_type}/{parent.task_name} ({parent.id}): "
+                        f"{exc}",
+                        "dynamic-data-failed",
+                    ) from exc
+
                 try:
                     orchestrator.callback(work_request)
                 except Exception as exc:
                     raise WorkflowRunError(
-                        work_request, f"orchestrator failed: {exc}"
+                        work_request,
+                        f"Orchestrator failed: {exc}",
+                        "orchestrator-failed",
                     ) from exc
 
                 work_request.mark_completed(WorkRequest.Results.SUCCESS)
 
             case (TaskTypes.WORKFLOW, _):
+                try:
+                    orchestrator.ensure_dynamic_data(
+                        TaskDatabase(orchestrator.work_request)
+                    )
+                except Exception as exc:
+                    raise WorkflowRunError(
+                        work_request,
+                        f"Failed to compute dynamic data: {exc}",
+                        "dynamic-data-failed",
+                    ) from exc
+
                 match work_request.status:
                     case (
                         WorkRequest.Statuses.RUNNING
@@ -434,15 +459,18 @@ def orchestrate_workflow(work_request: WorkRequest) -> None:
                     case _:
                         raise WorkflowRunError(
                             work_request,
-                            f"work request is in status {work_request.status}, "
+                            f"Work request is in status {work_request.status}, "
                             f"not running",
+                            "wrong-status",
                         )
 
                 try:
                     orchestrator.populate()
                 except Exception as exc:
                     raise WorkflowRunError(
-                        work_request, f"orchestrator failed: {exc}"
+                        work_request,
+                        f"Orchestrator failed: {exc}",
+                        "orchestrator-failed",
                     ) from exc
 
                 if work_request.children.exists():
@@ -468,5 +496,10 @@ def orchestrate_workflow(work_request: WorkRequest) -> None:
             exc.work_request.id,
             exc.message,
         )
-        work_request.mark_completed(WorkRequest.Results.ERROR)
+        work_request.mark_completed(
+            WorkRequest.Results.ERROR,
+            output_data=OutputData(
+                errors=[OutputDataError(message=exc.message, code=exc.code)]
+            ),
+        )
         raise
