@@ -8,10 +8,11 @@
 # contained in the LICENSE file.
 
 """Piuparts workflow."""
+
 from debusine.artifacts.models import ArtifactCategory
 from debusine.client.models import LookupChildType
 from debusine.server.collections.lookup import lookup_multiple
-from debusine.server.workflows import Workflow
+from debusine.server.workflows import Workflow, workflow_utils
 from debusine.server.workflows.models import (
     PiupartsWorkflowData,
     WorkRequestWorkflowData,
@@ -41,7 +42,9 @@ class PiupartsWorkflow(Workflow[PiupartsWorkflowData, BaseDynamicTaskData]):
         # piuparts will be run on the intersection of the provided
         # list of architectures (if any) and the architectures
         # provided in binary_artifacts
-        architectures = self.get_architectures(self.data.binary_artifacts)
+        architectures = workflow_utils.get_architectures(
+            self, self.data.binary_artifacts
+        )
         #  architectures: if set, only run on any of these architecture names
         if self.data.architectures is not None:
             architectures.intersection_update(self.data.architectures)
@@ -63,9 +66,12 @@ class PiupartsWorkflow(Workflow[PiupartsWorkflowData, BaseDynamicTaskData]):
 
         for architecture in sorted(architectures):
             # group arch-all + arch-any
-            arch_subset_binary_artifacts = self.filter_artifact_lookup_by_arch(
-                self.data.binary_artifacts,
-                (architecture, "all"),
+            arch_subset_binary_artifacts = (
+                workflow_utils.filter_artifact_lookup_by_arch(
+                    self,
+                    self.data.binary_artifacts,
+                    (architecture, "all"),
+                )
             )
 
             # The concrete architecture, or
@@ -102,10 +108,15 @@ class PiupartsWorkflow(Workflow[PiupartsWorkflowData, BaseDynamicTaskData]):
             # an architecture field in their data.
             self.requires_artifact(wr, arch_subset_binary_artifacts)
 
-    def compute_dynamic_data(
+    def build_dynamic_data(
         self, task_database: TaskDatabaseInterface  # noqa: U100
     ) -> BaseDynamicTaskData:
-        """Compute dynamic data for this workflow."""
+        """
+        Compute dynamic data for this workflow.
+
+        :subject: source package names of ``binary_artifacts`` separated
+          by spaces
+        """
         binaries = lookup_multiple(
             self.data.binary_artifacts,
             workflow_root=self.work_request.get_workflow_root(),
@@ -113,7 +124,7 @@ class PiupartsWorkflow(Workflow[PiupartsWorkflowData, BaseDynamicTaskData]):
             user=self.work_request.created_by,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
-        source_package_names = self.get_source_package_names(
+        source_package_names = workflow_utils.get_source_package_names(
             binaries,
             configuration_key="binary_artifacts",
             artifact_expected_categories=(
@@ -123,9 +134,7 @@ class PiupartsWorkflow(Workflow[PiupartsWorkflowData, BaseDynamicTaskData]):
             ),
         )
 
-        return BaseDynamicTaskData(
-            subject=" ".join(source_package_names),
-        )
+        return BaseDynamicTaskData(subject=" ".join(source_package_names))
 
     def get_label(self) -> str:
         """Return the task label."""
