@@ -9,15 +9,13 @@
 
 """qa workflow."""
 
-from functools import cached_property
-
-from debusine.artifacts import SourcePackage
-from debusine.artifacts.models import DebianSourcePackage
 from debusine.client.models import LookupChildType
-from debusine.db.models import Artifact
 from debusine.server.collections.lookup import lookup_single
-from debusine.server.workflows import Workflow
-from debusine.server.workflows.base import orchestrate_workflow
+from debusine.server.workflows import Workflow, workflow_utils
+from debusine.server.workflows.base import (
+    WorkflowValidationError,
+    orchestrate_workflow,
+)
 from debusine.server.workflows.models import (
     AutopkgtestWorkflowData,
     LintianWorkflowData,
@@ -42,27 +40,29 @@ class QAWorkflow(Workflow[QAWorkflowData, BaseDynamicTaskData]):
 
     TASK_NAME = "qa"
 
-    @cached_property
-    def source_package(self) -> Artifact:
-        """Source package artifact."""
-        artifact = lookup_single(
-            self.data.source_artifact,
-            self.workspace,
-            user=self.work_request.created_by,
-            expect_type=LookupChildType.ARTIFACT,
-        ).artifact
-        return self.locate_debian_source_package("source_artifact", artifact)
+    def validate_input(self) -> None:
+        """Thorough validation of input data."""
+        if self.data.reverse_dependencies_autopkgtest_suite is not None:
+            try:
+                lookup_single(
+                    self.data.reverse_dependencies_autopkgtest_suite,
+                    self.workspace,
+                    user=self.work_request.created_by,
+                    workflow_root=self.work_request.get_workflow_root(),
+                    expect_type=LookupChildType.COLLECTION,
+                ).collection
+            except LookupError as e:
+                raise WorkflowValidationError(str(e)) from e
 
-    @cached_property
-    def source_package_data(self) -> DebianSourcePackage:
-        """Parsed source package artifact data."""
-        return SourcePackage.create_data(self.source_package.data)
-
-    def compute_dynamic_data(
+    def build_dynamic_data(
         self, task_database: TaskDatabaseInterface  # noqa: U100
     ) -> BaseDynamicTaskData:
-        """Compute dynamic data for this workflow."""
-        source_data = self.source_package_data
+        """
+        Compute dynamic data for this workflow.
+
+        :subject: package name of ``source_artifact``
+        """
+        source_data = workflow_utils.source_package_data(self)
         return BaseDynamicTaskData(
             subject=source_data.name,
             parameter_summary=f"{source_data.name}_{source_data.version}",
@@ -73,8 +73,8 @@ class QAWorkflow(Workflow[QAWorkflowData, BaseDynamicTaskData]):
         if (data_archs := self.data.architectures) is not None:
             architectures = set(data_archs)
         else:
-            architectures = self.get_available_architectures(
-                vendor=self.data.vendor, codename=self.data.codename
+            architectures = workflow_utils.get_available_architectures(
+                self, vendor=self.data.vendor, codename=self.data.codename
             )
 
         if (
@@ -89,8 +89,10 @@ class QAWorkflow(Workflow[QAWorkflowData, BaseDynamicTaskData]):
 
         effective_architectures = sorted(architectures)
 
-        filtered_binary_artifacts = self.filter_artifact_lookup_by_arch(
-            self.data.binary_artifacts, architectures
+        filtered_binary_artifacts = (
+            workflow_utils.filter_artifact_lookup_by_arch(
+                self, self.data.binary_artifacts, architectures
+            )
         )
 
         if self.data.enable_autopkgtest:
