@@ -20,7 +20,11 @@ from debusine.artifacts.models import (
 from debusine.db.context import context
 from debusine.db.models import Artifact, TaskDatabase, WorkRequest
 from debusine.server.collections.debian_suite import DebianSuiteManager
-from debusine.server.workflows import QAWorkflow, workflow_utils
+from debusine.server.workflows import (
+    QAWorkflow,
+    WorkflowValidationError,
+    workflow_utils,
+)
 from debusine.server.workflows.base import orchestrate_workflow
 from debusine.server.workflows.models import (
     BaseWorkflowData,
@@ -45,7 +49,10 @@ class QAWorkflowTests(TestCase):
     """Unit tests for :py:class:`QAWorkflow`."""
 
     def create_qa_workflow(
-        self, *, extra_task_data: dict[str, Any]
+        self,
+        *,
+        extra_task_data: dict[str, Any] | None = None,
+        validate: bool = True,
     ) -> QAWorkflow:
         """Create a qa workflow."""
         task_data = {
@@ -54,10 +61,10 @@ class QAWorkflowTests(TestCase):
             "vendor": "debian",
             "codename": "bookworm",
         }
-        task_data.update(extra_task_data)
+        if extra_task_data is not None:
+            task_data.update(extra_task_data)
         wr = self.playground.create_workflow(
-            task_name="qa",
-            task_data=task_data,
+            task_name="qa", task_data=task_data, validate=validate
         )
         return QAWorkflow(wr)
 
@@ -149,6 +156,30 @@ class QAWorkflowTests(TestCase):
         )
         return artifact
 
+    def test_validate_input(self) -> None:
+        """validate_input passes a valid case."""
+        workflow = self.create_qa_workflow()
+
+        workflow.validate_input()
+
+    def test_validate_input_bad_rdep_autopkgtest_suite(self) -> None:
+        """validate_input raises errors in looking up a suite."""
+        workflow = self.create_qa_workflow(
+            extra_task_data={
+                "enable_reverse_dependencies_autopkgtest": True,
+                "reverse_dependencies_autopkgtest_suite": (
+                    "nonexistent@debian:suite"
+                ),
+            },
+            validate=False,
+        )
+
+        with self.assertRaisesRegex(
+            WorkflowValidationError,
+            "'nonexistent@debian:suite' does not exist or is hidden",
+        ):
+            workflow.validate_input()
+
     def test_compute_dynamic_data(self) -> None:
         source_artifact = self.playground.create_source_artifact(
             name="hello", version="1.0-1"
@@ -700,5 +731,5 @@ class QAWorkflowTests(TestCase):
 
     def test_get_label(self) -> None:
         """Test get_label()."""
-        w = self.create_qa_workflow(extra_task_data={})
+        w = self.create_qa_workflow()
         self.assertEqual(w.get_label(), "run QA")
