@@ -126,8 +126,6 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
     @context.disable_permission_checks()
     def test_validate_input(self) -> None:
         """Test that validate_input passes a valid case."""
-        user = self.playground.get_default_user()
-
         tarball_item = self.playground.create_debian_environment()
 
         source = self.create_source_package(
@@ -135,16 +133,15 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
         )
 
         template = self.create_sbuild_template()
-        wr = WorkRequest.objects.create_workflow(
-            template=template,
-            data={
+        wr = self.playground.create_workflow(
+            template,
+            task_data={
                 "input": {
                     "source_artifact": source.pk,
                 },
                 "target_distribution": "debian:bookworm",
                 "architectures": ["all", "amd64"],
             },
-            created_by=user,
         )
         self.assertEqual(wr.status, WorkRequest.Statuses.PENDING)
 
@@ -167,9 +164,9 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             variables={"variant": "buildd", "backend": "unshare"},
         )
 
-        wr = WorkRequest.objects.create_workflow(
-            template=template,
-            data={
+        wr = self.playground.create_workflow(
+            template,
+            task_data={
                 "input": {
                     "source_artifact": source.pk,
                 },
@@ -177,7 +174,6 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                 "architectures": ["all", "amd64"],
                 "environment_variant": "buildd",
             },
-            created_by=user,
         )
         o = SbuildWorkflow(wr)
         self.assertEqual(o._get_environment("amd64"), tarball_item.artifact)
@@ -186,8 +182,6 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
     @context.disable_permission_checks()
     def test_computation_of_architecture_intersection(self) -> None:
         """Test computation of architectures to build."""
-        user = self.playground.get_default_user()
-
         template = self.create_sbuild_template()
 
         dsc_archs = {
@@ -210,16 +204,15 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             for wr_archs in (["amd64"], ["all"], ["arm64", "all"]):
                 with self.subTest("", dsc_arch=dsc_arch, wr_archs=wr_archs):
                     o = SbuildWorkflow(
-                        WorkRequest.objects.create_workflow(
-                            template=template,
-                            data={
+                        self.playground.create_workflow(
+                            template,
+                            task_data={
                                 "input": {
                                     "source_artifact": source.pk,
                                 },
                                 "target_distribution": "debian:bookworm",
                                 "architectures": wr_archs,
                             },
-                            created_by=user,
                             validate=False,
                         ),
                     )
@@ -270,9 +263,9 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             )
 
             template = self.create_sbuild_template()
-            wr = WorkRequest.objects.create_workflow(
-                template=template,
-                data={
+            wr = self.playground.create_workflow(
+                template,
+                task_data={
                     "input": {
                         "source_artifact": source.pk,
                     },
@@ -280,7 +273,6 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                     "target_distribution": "debian:bookworm",
                     "architectures": ["all", "amd64", "s390x"],
                 },
-                created_by=self.playground.get_default_user(),
             )
             self.assertEqual(wr.status, WorkRequest.Statuses.PENDING)
             wr.mark_running()
@@ -407,9 +399,9 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                 "hello", "1.0", dsc_fields={"Architecture": "all amd64 s390x"}
             )
             template = self.create_sbuild_template()
-            wr = WorkRequest.objects.create_workflow(
-                template=template,
-                data={
+            wr = self.playground.create_workflow(
+                template,
+                task_data={
                     "input": {
                         "source_artifact": source.pk,
                     },
@@ -418,7 +410,6 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                     "architectures": ["all", "amd64", "s390x"],
                     "arch_all_host_architecture": "s390x",
                 },
-                created_by=self.playground.get_default_user(),
             )
             self.assertEqual(wr.status, WorkRequest.Statuses.PENDING)
             wr.mark_running()
@@ -487,15 +478,14 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             )
 
             template = self.create_sbuild_template()
-            wr = WorkRequest.objects.create_workflow(
-                template=template,
-                data={
+            wr = self.playground.create_workflow(
+                template,
+                task_data={
                     "input": {"source_artifact": source.pk},
                     "backend": "unshare",
                     "target_distribution": "debian:bookworm",
                     "architectures": ["amd64"],
                 },
-                created_by=self.playground.get_default_user(),
             )
             self.assertEqual(wr.status, WorkRequest.Statuses.PENDING)
             wr.mark_running()
@@ -617,16 +607,15 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                 },
             )
             template = self.create_sbuild_template()
-            wr = WorkRequest.objects.create_workflow(
-                template=template,
-                data={
+            wr = self.playground.create_workflow(
+                template,
+                task_data={
                     "input": {"source_artifact": source.pk},
                     "backend": "unshare",
                     "target_distribution": "debian:bookworm",
                     "architectures": ["amd64"],
                     "retry_delays": ["1h", "1d"],
                 },
-                created_by=self.playground.get_default_user(),
             )
             self.assertEqual(wr.status, WorkRequest.Statuses.PENDING)
             wr.mark_running()
@@ -697,16 +686,15 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                 },
             )
             template = self.create_sbuild_template()
-            wr = WorkRequest.objects.create_workflow(
-                template=template,
-                data={
+            wr = self.playground.create_workflow(
+                template,
+                task_data={
                     "input": {"source_artifact": source.pk},
                     "backend": "unshare",
                     "target_distribution": "debian:bookworm",
                     "architectures": ["amd64"],
                     "signing_template_names": {"amd64": ["hello-unsigned"]},
                 },
-                created_by=self.playground.get_default_user(),
             )
             self.assertEqual(wr.status, WorkRequest.Statuses.PENDING)
             wr.mark_running()
@@ -755,10 +743,9 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             architectures: list[str] = ["amd64"],
             **kwargs: Any,
         ) -> SbuildWorkflow:
-            user = self.playground.get_default_user()
-            workflow = WorkRequest.objects.create_workflow(
-                template=template,
-                data={
+            workflow = self.playground.create_workflow(
+                template,
+                task_data={
                     "input": {
                         "source_artifact": source.pk,
                     },
@@ -766,7 +753,6 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                     "architectures": architectures,
                     **kwargs,
                 },
-                created_by=user,
                 validate=False,
             )
             orchestrator = SbuildWorkflow(workflow)
@@ -869,7 +855,7 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
         """Test get_label."""
         wr = WorkRequest(
             task_data={
-                "prefix": "signed-source/",
+                "prefix": "signed-source|",
                 "input": {
                     "source_artifact": 42,
                 },
@@ -879,4 +865,4 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             workspace=default_workspace(),
         )
         w = SbuildWorkflow(wr)
-        self.assertEqual(w.get_label(), "signed-source/build a package")
+        self.assertEqual(w.get_label(), "signed-source|build a package")
