@@ -96,10 +96,19 @@ class Lintian(
         ):
             self._severities_to_level[severity] = ranking
 
-    def compute_dynamic_data(
+    def build_dynamic_data(
         self, task_database: TaskDatabaseInterface
     ) -> LintianDynamicData:
-        """Resolve artifact lookups for this task."""
+        """
+        Resolve artifact lookups for this task.
+
+        :subject: name of source package of ``input.source_artifact`` or
+          ``input.binary_artifacts``
+        :runtime_context: ``what:distribution`` where ``what`` is what
+          got analyzed (``source``, ``binary-all`` or ``binary-any``) and
+          ``distribution`` is ``codename`` of ``environment``.
+        :configuration_context: ``codename`` of ``environment``
+        """
         input_source_artifact = task_database.lookup_single_artifact(
             self.data.input.source_artifact
         )
