@@ -18,6 +18,7 @@ import io
 import itertools
 import unittest
 from collections.abc import Callable, Collection, Generator
+from pathlib import Path
 from typing import (
     Any,
     ClassVar,
@@ -36,6 +37,7 @@ import django.test
 import requests
 from asgiref.sync import async_to_sync
 from channels.layers import get_channel_layer
+from django.conf import settings
 from django.db import transaction
 from django.db.models import Model, QuerySet
 from django.http.response import HttpResponseBase
@@ -229,6 +231,18 @@ class BaseDjangoTestCase(django.test.SimpleTestCase, BaseTestCase):
     tests that do depend on Django code.
     """
 
+    @classmethod
+    def setUpClass(cls) -> None:
+        """
+        Reset context before the TestCase initializes.
+
+        Context is likely to be accessed by class-level test case setup (like
+        ``setUpTestData``), and a context left over from a previous test will
+        interfere with that
+        """
+        context.reset()
+        super().setUpClass()
+
     def setUp(self) -> None:
         """Ensure test methods do not change the global scope."""
         super().setUp()
@@ -466,6 +480,30 @@ class BaseDjangoTestCase(django.test.SimpleTestCase, BaseTestCase):
                     print(f"* {error}", file=buf)
                 self.fail(buf.getvalue())
 
+    @contextlib.contextmanager
+    def assert_model_count_unchanged(
+        self, model_cls: type[Model]
+    ) -> Generator[None, None, None]:
+        """Context manager to assert count of rows in a model is unchanged."""
+        initial_count = getattr(model_cls, "objects").count()
+        try:
+            yield
+        finally:
+            self.assertEqual(
+                getattr(model_cls, "objects").count(), initial_count
+            )
+
+    @contextlib.contextmanager
+    def template_dir(self) -> Generator[Path, None, None]:
+        """Prepend a temporary directory to the template search paths."""
+        template_dir = self.create_temporary_directory()
+        template_settings = copy.deepcopy(settings.TEMPLATES)
+        cast(list[str], template_settings[0]["DIRS"]).insert(
+            0, template_dir.as_posix()
+        )
+        with django.test.override_settings(TEMPLATES=template_settings):
+            yield template_dir
+
 
 class PlaygroundTestCase(BaseDjangoTestCase):
     """
