@@ -30,6 +30,9 @@ class Icons(enum.StrEnum):
     COLLECTIONS = "stack"
     USER = "person-circle"
     WORKER = "pc-horizontal"
+    WORKER_EXTERNAL = "pc-horizontal"
+    WORKER_CELERY = "gear"
+    WORKER_SIGNING = "patch-check"
     WORKFLOW = "diagram-3"
     WORK_REQUEST = "hammer"
     WORK_REQUEST_STATUS = "heart-pulse"
