@@ -21,7 +21,7 @@ from django.template.base import FilterExpression, Parser, Token
 from django.utils.html import format_html
 from django.utils.safestring import SafeString
 
-import debusine.web.views.utils
+import debusine.web.views.view_utils
 from debusine.db.context import context
 from debusine.server.scopes import urlconf_scope
 from debusine.web.icons import Icons
@@ -128,7 +128,7 @@ def has_perm(resource: Model, name: str) -> bool:
 @register.filter
 def format_yaml(value: Any) -> str:
     """Format a data structure as YAML."""
-    return debusine.web.views.utils.format_yaml(value)
+    return debusine.web.views.view_utils.format_yaml(value)
 
 
 @register.simple_tag
