@@ -35,6 +35,7 @@ from debusine.web.views.base import (
     BaseUIView,
     CreateViewBase,
     DetailViewBase,
+    SingleObjectMixinBase,
     WorkspaceView,
 )
 from debusine.web.views.base_rightbar import RightbarUIView
@@ -88,8 +89,7 @@ class ArtifactView(WorkspaceView, RightbarUIView, abc.ABC):
 
 
 class ArtifactDetailView(
-    ArtifactView,
-    DetailViewBase[Artifact],
+    ArtifactView, DetailViewBase[Artifact], RightbarUIView
 ):
     """Display an artifact and its file(s)."""
 
@@ -183,31 +183,38 @@ class ArtifactDetailView(
         return ctx
 
 
-class ArtifactDetailFileView(
+class FileView(
+    PathMixin,
+    FileDownloadMixin,
     ArtifactView,
-    DetailViewBase[FileInArtifact],
+    SingleObjectMixinBase[FileInArtifact],
 ):
-    """Display a file from an artifact."""
+    """Base for all FileInArtifact views."""
 
     model = FileInArtifact
-    template_name = "web/fileinartifact-detail.html"
-    permission_denied_message = (
-        "Non-public artifact: you might need to login "
-        "or make a request with a valid Token header"
-    )
-    pk_url_kwarg = "file_in_artifact_id"
+    object: FileInArtifact
 
     def get_queryset(self) -> QuerySet[FileInArtifact]:
         """Get the queryset."""
         return (
             super()
             .get_queryset()
-            .filter(
-                artifact_id=self.kwargs["artifact_id"], path=self.kwargs["path"]
-            )
+            .filter(artifact_id=self.kwargs["artifact_id"])
             .select_related("artifact__workspace")
         )
 
+    def get_object(
+        self, queryset: QuerySet[FileInArtifact] | None = None
+    ) -> FileInArtifact:
+        """Return the FileInArtifact object to show."""
+        # Prevent repeated queries: get_artifact calls get_object before
+        # dispatch, and Django's BaseDetailView calls get_object in get()
+        if not hasattr(self, "object"):
+            assert queryset is None
+            queryset = self.get_queryset()
+            self.object = get_object_or_404(queryset, path=self.kwargs["path"])
+        return self.object
+
     def get_artifact(self) -> Artifact:
         """Artifact for this view."""
         return self.get_object().artifact
@@ -236,29 +243,23 @@ class ArtifactDetailFileView(
         return ctx
 
 
-class ArtifactDetailFileViewRaw(
-    PathMixin,
-    FileDownloadMixin,
-    ArtifactView,
-):
-    """Download a file from an artifact, displayed inline."""
+class FileDetailView(FileView, DetailViewBase[FileInArtifact]):
+    """Display a file from an artifact."""
 
-    def get_artifact(self) -> Artifact:
-        """Return the current artifact."""
-        self.file_in_artifact = get_object_or_404(
-            FileInArtifact.objects.select_related("artifact__workspace"),
-            artifact_id=self.kwargs["artifact_id"],
-            id=self.kwargs["file_in_artifact_id"],
-            path=self.path.rstrip("/"),
-        )
-        return self.file_in_artifact.artifact
+    template_name = "web/fileinartifact-detail.html"
+    object: FileInArtifact
+
+
+class FileDetailViewRaw(FileView):
+    """Download a file from an artifact, displayed inline."""
 
     def get(
         self, request: HttpRequest, **kwargs: Any  # noqa: U100
     ) -> HttpResponseBase:
         """Download file with disposition inline."""
-        ui_info = FileUI.from_file_in_artifact(self.file_in_artifact)
-        return self.stream_file(self.file_in_artifact, ui_info, download=False)
+        self.object = self.get_object()
+        ui_info = FileUI.from_file_in_artifact(self.object)
+        return self.stream_file(self.object, ui_info, download=False)
 
 
 class DownloadFormat(StrEnum):
@@ -406,6 +407,7 @@ class CreateArtifactView(
 
     template_name = "web/artifact-create.html"
     form_class = ArtifactForm
+    title = "Create artifact"
 
     def init_view(self) -> None:
         """Check can_create_artifacts permission."""
