@@ -8,7 +8,6 @@
 # contained in the LICENSE file.
 
 """Tests for the auth views."""
-import re
 from datetime import timedelta
 from textwrap import dedent
 from typing import Any, ClassVar
@@ -34,6 +33,7 @@ from debusine.test.django import (
     TestResponseType,
     override_permission,
 )
+from debusine.web.views.auth import UserDetailView
 from debusine.web.views.tests.utils import (
     ViewTestMixin,
     date_format,
@@ -111,6 +111,12 @@ class UserDetailViewTests(ViewTestMixin, TestCase):
 
     scenario = scenarios.DefaultScopeUser()
 
+    def test_get_title(self) -> None:
+        """Test get_title method."""
+        view = UserDetailView()
+        view.object = self.scenario.user
+        self.assertEqual(view.get_title(), self.scenario.user.get_full_name())
+
     def test_anonymous(self) -> None:
         """Test accessing as anonymous user."""
         response = self.client.get(self.scenario.user.get_absolute_url())
@@ -123,8 +129,7 @@ class UserDetailViewTests(ViewTestMixin, TestCase):
         """Test viewing the default user."""
         self.client.force_login(self.scenario.user)
         response = self.client.get(self.scenario.user.get_absolute_url())
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
 
         dl = self.assertHasElement(tree, "//dl[@id='person-info']")
         self.assertTextContentEqual(
@@ -156,8 +161,7 @@ class UserDetailViewTests(ViewTestMixin, TestCase):
             )
 
         response = self.client.get(self.scenario.user.get_absolute_url())
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
 
         table = self.assertHasElement(tree, "//table[@id='person-groups']")
         tbody = self.assertHasElement(table, "tbody")
@@ -180,8 +184,7 @@ class UserDetailViewTests(ViewTestMixin, TestCase):
         )
 
         response = self.client.get(self.scenario.user.get_absolute_url())
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
 
         table = self.assertHasElement(tree, "//table[@id='person-identities']")
         tr = self.assertHasElement(table, "tbody/tr")
@@ -204,8 +207,7 @@ class UserDetailViewTests(ViewTestMixin, TestCase):
         )
 
         response = self.client.get(user.get_absolute_url())
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertFalse(tree.xpath("//table[@id='person-identities']"))
 
     def test_detail_identities_without_provider(self) -> None:
@@ -218,8 +220,7 @@ class UserDetailViewTests(ViewTestMixin, TestCase):
             subject="playground@debian.org",
         )
         response = self.client.get(self.scenario.user.get_absolute_url())
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertFalse(tree.xpath("//table[@id='person-identities']"))
 
     @override_settings(SIGNON_PROVIDERS=[Provider(name="salsa", label="Salsa")])
@@ -233,8 +234,7 @@ class UserDetailViewTests(ViewTestMixin, TestCase):
             subject="playground@debian.org",
         )
         response = self.client.get(self.scenario.user.get_absolute_url())
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertFalse(tree.xpath("//table[@id='person-identities']"))
 
 
@@ -252,7 +252,7 @@ class UserTokenListViewTests(ViewTestMixin, TestCase):
                 kwargs={"username": self.scenario.user.username},
             )
         )
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         sample_cfg = self.assertHasElement(tree, "//pre[@id='example-config']")
         fqdn = settings.DEBUSINE_FQDN
         self.assertTextContentEqual(
@@ -281,9 +281,10 @@ class UserTokenListViewTests(ViewTestMixin, TestCase):
                 kwargs={"username": self.scenario.user.username},
             )
         )
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         sample_cfg = self.assertHasElement(tree, "//pre[@id='example-config']")
         fqdn = settings.DEBUSINE_FQDN
+        scope_list = " or ".join(str(s) for s in response.context["scopes"])
         self.assertTextContentEqual(
             sample_cfg,
             dedent(
@@ -293,14 +294,11 @@ class UserTokenListViewTests(ViewTestMixin, TestCase):
 
                 [server:{fqdn}]
                 api-url = https://{fqdn}/api
-                scope = one-of-the-scopes-listed-below
+                scope = {scope_list}
                 token = generated-token-goes-here
                 """
             ),
         )
-        ul = self.assertHasElement(tree, "//ul[@id='scope-list']")
-        self.assertTextContentEqual(ul.li[0], "debusine")
-        self.assertTextContentEqual(ul.li[1], "test")
 
     def test_logged_no_tokens(self) -> None:
         """Logged in user with no tokens."""
@@ -316,7 +314,7 @@ class UserTokenListViewTests(ViewTestMixin, TestCase):
                 kwargs={"username": self.scenario.user.username},
             )
         )
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertNavHasUser(tree, self.scenario.user)
 
         self.assertContains(
@@ -421,7 +419,7 @@ class UserTokenListViewTests(ViewTestMixin, TestCase):
         )
 
 
-class UserTokenCreateViewTests(TestCase):
+class UserTokenCreateViewTests(ViewTestMixin, TestCase):
     """Tests for UserTokenCreateView."""
 
     user: ClassVar[User]
@@ -443,31 +441,25 @@ class UserTokenCreateViewTests(TestCase):
                 "user:token-create",
                 kwargs={"username": self.user.username},
             ),
-            {"comment": comment},
+            {"comment": comment, "enabled": "true"},
         )
+        # This post, even if successful, renders a template and does not
+        # redirect
+        tree = self.assertResponseHTML(response)
 
-        # The token got created...
+        # The token got created, enabled, with user assigned
         token = Token.objects.get(comment=comment)
-
-        # and has the user assigned
         self.assertEqual(token.user, self.user)
+        self.assertTrue(token.enabled)
 
-        # check the returned token matches one in the db
-        token_key_re = re.compile(
-            r".*<code>(?P<token>[a-z0-9]*).*</code>.*",
-            re.DOTALL,
-        )
-        match = token_key_re.match(response.content.decode())
-        assert match is not None
-        resp_key = match.group("token")
-        self.assertEqual(token, Token.objects.get_token_or_none(resp_key))
+        # No expiration, no use yet
+        self.assertIsNone(token.expire_at)
+        self.assertIsNone(token.last_seen_at)
 
-        self.assertContains(
-            response,
-            _html_collapse_token(resp_key),
-            status_code=status.HTTP_200_OK,
-            html=True,
-        )
+        # Ensure token key shown matches the one of the new token
+        key_element = self.assertHasElement(tree, "//code[@id='token-key']")
+        key = self.get_node_text_normalized(key_element)
+        self.assertEqual(token, Token.objects.get_token_or_none(key))
 
     def test_no_logged(self) -> None:
         """Request with a non-logged user: cannot create a token."""
@@ -811,7 +803,7 @@ class UserGroupListViewTests(ViewTestMixin, TestCase):
     def test_get_self_no_groups(self) -> None:
         """Get one's own empty group list."""
         response = self.assertGetSucceeds(self.scenario.user)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         intro = self.assertHasElement(tree, "//p[@id='no-groups-message']")
         self.assertTextContentEqual(
             intro, "You are not currently a member of any group."
@@ -824,7 +816,7 @@ class UserGroupListViewTests(ViewTestMixin, TestCase):
             response = self.assertGetSucceeds(
                 self.scenario.user, visitor=manager
             )
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         intro = self.assertHasElement(tree, "//p[@id='no-groups-message']")
         self.assertTextContentEqual(
             intro,
@@ -837,7 +829,7 @@ class UserGroupListViewTests(ViewTestMixin, TestCase):
         """Get one's own group list."""
         self.playground.add_user(self.group, self.scenario.user)
         response = self.assertGetSucceeds(self.scenario.user)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         intro = self.assertHasElement(tree, "//p[@id='intro']")
         self.assertTextContentEqual(
             intro, "This page lists all groups of which you are a member."
@@ -856,7 +848,7 @@ class UserGroupListViewTests(ViewTestMixin, TestCase):
         self.playground.add_user(self.group, self.scenario.user)
         with override_permission(Group, "can_display", DenyAll):
             response = self.assertGetSucceeds(self.scenario.user)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertHasElement(tree, "//p[@id='no-groups-message']")
 
     def test_get_visitor(self) -> None:
@@ -867,7 +859,7 @@ class UserGroupListViewTests(ViewTestMixin, TestCase):
             response = self.assertGetSucceeds(
                 self.scenario.user, visitor=manager
             )
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         intro = self.assertHasElement(tree, "//p[@id='intro']")
         self.assertTextContentEqual(
             intro,
@@ -1016,7 +1008,7 @@ class GroupDetailViewTests(ViewTestMixin, TestCase):
         """View the page as a group member."""
         self.playground.add_user(self.group, self.other_user)
         response = self.assertGetSucceeds(self.group)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         info_role = self.assertHasElement(tree, "//p[@id='info-role']")
         self.assertTextContentEqual(
             info_role, "You are not a member of this group."
@@ -1031,7 +1023,7 @@ class GroupDetailViewTests(ViewTestMixin, TestCase):
             self.group.ephemeral = True
             self.group.save()
         response = self.assertGetSucceeds(self.group)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertEphemeral(tree, True)
 
     def test_get_as_member(self) -> None:
@@ -1039,7 +1031,7 @@ class GroupDetailViewTests(ViewTestMixin, TestCase):
         self.playground.add_user(self.group, self.scenario.user)
         self.playground.add_user(self.group, self.other_user)
         response = self.assertGetSucceeds(self.group)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         info_role = self.assertHasElement(tree, "//p[@id='info-role']")
         self.assertTextContentEqual(info_role, "You have role member.")
         self.assertEphemeral(tree, False)
@@ -1059,7 +1051,7 @@ class GroupDetailViewTests(ViewTestMixin, TestCase):
         )
         self.playground.add_user(self.group, self.other_user)
         response = self.assertGetSucceeds(self.group)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         info_role = self.assertHasElement(tree, "//p[@id='info-role']")
         self.assertTextContentEqual(info_role, "You have role admin.")
         self.assertEphemeral(tree, False)
@@ -1098,8 +1090,7 @@ class GroupDetailViewTests(ViewTestMixin, TestCase):
         response = self.client.post(
             self.group.get_absolute_url(), data={"username": "does-not-exist"}
         )
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertEphemeral(tree, False)
         self.assertFormErrors(tree, True)
 
@@ -1152,10 +1143,10 @@ class GroupDetailViewMemberPaginationTests(ViewTestMixin, TestCase):
     def test_default(self) -> None:
         """Test default view with no page or ordering selected."""
         response = self._get()
+        tree = self.assertResponseHTML(response)
         ctx = response.context
         self.assertEqual(ctx["asc"], "1")
         self.assertIsNone(ctx["order"])
-        tree = self.assertHTMLValid(response)
         self.assertGroupTable(
             tree,
             [
@@ -1175,10 +1166,10 @@ class GroupDetailViewMemberPaginationTests(ViewTestMixin, TestCase):
     def test_page(self) -> None:
         """Test selecting the page number."""
         response = self._get(page=2)
+        tree = self.assertResponseHTML(response)
         ctx = response.context
         self.assertEqual(ctx["asc"], "1")
         self.assertIsNone(ctx["order"])
-        tree = self.assertHTMLValid(response)
         self.assertGroupTable(
             tree,
             [
@@ -1198,10 +1189,10 @@ class GroupDetailViewMemberPaginationTests(ViewTestMixin, TestCase):
     def test_order_username(self) -> None:
         """Test ordering by username."""
         response = self._get(order="name")
+        tree = self.assertResponseHTML(response)
         ctx = response.context
         self.assertEqual(ctx["asc"], "1")
         self.assertEqual(ctx["order"], "name")
-        tree = self.assertHTMLValid(response)
         self.assertGroupTable(
             tree,
             [
@@ -1219,10 +1210,10 @@ class GroupDetailViewMemberPaginationTests(ViewTestMixin, TestCase):
         )
 
         response = self._get(order="name", asc="0")
+        tree = self.assertResponseHTML(response)
         ctx = response.context
         self.assertEqual(ctx["asc"], "0")
         self.assertEqual(ctx["order"], "name")
-        tree = self.assertHTMLValid(response)
         self.assertGroupTable(
             tree,
             [
@@ -1242,10 +1233,10 @@ class GroupDetailViewMemberPaginationTests(ViewTestMixin, TestCase):
     def test_order_role(self) -> None:
         """Test ordering by role."""
         response = self._get(order="role")
+        tree = self.assertResponseHTML(response)
         ctx = response.context
         self.assertEqual(ctx["asc"], "1")
         self.assertEqual(ctx["order"], "role")
-        tree = self.assertHTMLValid(response)
         self.assertGroupTable(
             tree,
             [
@@ -1263,10 +1254,10 @@ class GroupDetailViewMemberPaginationTests(ViewTestMixin, TestCase):
         )
 
         response = self._get(order="role", asc="0")
+        tree = self.assertResponseHTML(response)
         ctx = response.context
         self.assertEqual(ctx["asc"], "0")
         self.assertEqual(ctx["order"], "role")
-        tree = self.assertHTMLValid(response)
         self.assertGroupTable(
             tree,
             [
@@ -1362,8 +1353,7 @@ class MembershipUpdateViewTests(ViewTestMixin, TestCase):
         response = self._post(
             self.scenario.user, data={"role": "does-not-exist"}
         )
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         errorlist = self.assertHasElement(tree, "//ul[@class='errorlist']")
         self.assertEqual(len(errorlist.li), 1)
         self.assertTextContentEqual(
@@ -1375,8 +1365,7 @@ class MembershipUpdateViewTests(ViewTestMixin, TestCase):
     def test_intro_self(self) -> None:
         """Check page introduction for oneself."""
         response = self._get(self.scenario.user)
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         intro = self.assertHasElement(tree, "//p[@id='intro']")
         self.assertTextContentEqual(
             intro,
@@ -1386,8 +1375,7 @@ class MembershipUpdateViewTests(ViewTestMixin, TestCase):
     def test_intro_other(self) -> None:
         """Check page introduction for another user."""
         response = self._get(self.other_user)
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         intro = self.assertHasElement(tree, "//p[@id='intro']")
         self.assertTextContentEqual(
             intro,
@@ -1516,8 +1504,7 @@ class MembershipDeleteViewTests(ViewTestMixin, TestCase):
     def test_intro_self(self) -> None:
         """Check page introduction for oneself."""
         response = self._get(self.scenario.user)
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         intro = self.assertHasElement(tree, "//p[@id='intro']")
         self.assertTextContentEqual(
             intro,
@@ -1528,8 +1515,7 @@ class MembershipDeleteViewTests(ViewTestMixin, TestCase):
     def test_intro_other(self) -> None:
         """Check page introduction for another user."""
         response = self._get(self.other_user)
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         intro = self.assertHasElement(tree, "//p[@id='intro']")
         self.assertTextContentEqual(
             intro,
@@ -1622,7 +1608,7 @@ class GroupAuditLogViewTests(ViewTestMixin, TestCase):
     def test_get_as_nonmember(self) -> None:
         """View the page as a group member."""
         response = self.assertGetSucceeds()
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         rows = self.assertHasRows(tree)
         self.assertEqual(rows, [5, 4, 3, 2, 1, 0])
 
@@ -1630,7 +1616,7 @@ class GroupAuditLogViewTests(ViewTestMixin, TestCase):
         """View the page as a group member."""
         self.playground.add_user(self.group, self.scenario.user)
         response = self.assertGetSucceeds()
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         rows = self.assertHasRows(tree)
         self.assertEqual(rows, [5, 4, 3, 2, 1, 0])
 
@@ -1640,19 +1626,6 @@ class GroupAuditLogViewTests(ViewTestMixin, TestCase):
             self.group, self.scenario.user, Group.Roles.ADMIN
         )
         response = self.assertGetSucceeds()
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         rows = self.assertHasRows(tree)
         self.assertEqual(rows, [5, 4, 3, 2, 1, 0])
-
-
-def _html_collapse_token(key: str) -> str:
-    """Return HTML for a show/hide button for token key."""
-    return f'''<button class="btn btn-primary btn-sm" type="button"
-        data-bs-toggle="collapse"
-            data-bs-target="#{key}Div">
-        Show / Hide Token
-        </button>
-        <div class="collapse" id="{key}Div">
-            <code>{key}</code>
-        </div>
-    '''
