@@ -11,7 +11,6 @@
 
 import textwrap
 from datetime import timedelta
-from types import GeneratorType
 from typing import Any, ClassVar
 
 import lxml
@@ -49,6 +48,8 @@ from debusine.tasks.models import (
     MmDebstrapBootstrapOptions,
     MmDebstrapData,
     NoopData,
+    OutputData,
+    OutputDataError,
     SystemBootstrapRepository,
     TaskTypes,
 )
@@ -56,16 +57,12 @@ from debusine.test.django import AllowAll, TestCase, override_permission
 from debusine.web.templatetags.artifacts import artifact_category_label
 from debusine.web.views import ui_shortcuts
 from debusine.web.views.lintian import LintianView
-from debusine.web.views.tests.test_views import _sort_table_handle
 from debusine.web.views.tests.utils import ViewTestMixin
-from debusine.web.views.work_request import (
-    WorkRequestDetailView,
-    WorkRequestListView,
-)
+from debusine.web.views.work_request import WorkRequestDetailView
 
 
 class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
-    """Tests for WorkRequestDetailView class."""
+    """Tests for :py:class:`WorkRequestDetailView`."""
 
     scenario = scenarios.DefaultContext()
     source: ClassVar[Artifact]
@@ -165,6 +162,15 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
         else:
             return yaml.safe_load("".join(metadata.div[1].itertext()))
 
+    def test_get_title(self) -> None:
+        """Test get_title method."""
+        view = WorkRequestDetailView()
+        view.object = self.work_request
+        self.assertEqual(
+            view.get_title(),
+            f"{self.work_request.pk}: {self.work_request.get_label()}",
+        )
+
     def test_get_work_request(self) -> None:
         """View detail return work request information."""
         with context.disable_permission_checks():
@@ -173,9 +179,7 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
             artifact.save()
 
         response = self.client.get(self.work_request.get_absolute_url())
-
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
 
         sidebar = response.context["sidebar_items"]
         self.assertEqual(sidebar[0].value, "build hello")
@@ -220,10 +224,9 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
         self.work_request.save()
 
         response = self.client.get(self.work_request.get_absolute_url())
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        tree = self.assertResponseHTML(response)
         self.assertIn("task_data_original", response.context)
         self.assertTrue(response.context["task_data_configured"])
-        tree = self.assertHTMLValid(response)
         configured, orig = self.assertHasMetadata(tree)
         self.assertEqual(configured["host_architecture"], "arm64")
         self.assertEqual(orig["host_architecture"], "amd64")
@@ -239,9 +242,7 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
         artifact.save()
 
         response = self.client.get(self.work_request.get_absolute_url())
-
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertArtifacts(tree, [], [artifact])
 
     def test_get_work_request_private_workspace_unauthenticated(self) -> None:
@@ -353,6 +354,31 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
             ["10\xa0minutes"],
         )
 
+    def test_get_work_request_no_output_data_errors(self) -> None:
+        """If the work request has no output data errors, they are not shown."""
+        response = self.client.get(self.work_request.get_absolute_url())
+        tree = self.assertResponseHTML(response)
+        self.assertFalse(tree.xpath("//ul[@id='errors']"))
+        self.assertNotContains(response, "<h2>Errors</h2>")
+
+    def test_get_work_request_output_data_errors(self) -> None:
+        """If the work request has output data errors, they are shown."""
+        self.work_request.result = WorkRequest.Results.ERROR
+        self.work_request.output_data = OutputData(
+            errors=[
+                OutputDataError(message="One error", code=""),
+                OutputDataError(message="Another error", code=""),
+            ]
+        )
+        self.work_request.save()
+
+        response = self.client.get(self.work_request.get_absolute_url())
+        tree = self.assertResponseHTML(response)
+        errors = tree.xpath("//ul[@id='errors']")[0]
+        self.assertEqual(len(errors.li), 2)
+        self.assertTextContentEqual(errors.li[0], "One error")
+        self.assertTextContentEqual(errors.li[1], "Another error")
+
     @context.disable_permission_checks()
     def make_work_request_lintian(self, work_request: WorkRequest) -> None:
         """Change work_request to "lintian", create source artifact."""
@@ -492,9 +518,7 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
         self.work_request.save()
 
         response = self.client.get(self.work_request.get_absolute_url())
-
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         card = tree.xpath("//div[@id='metadata']")[0]
 
         self.assertTextContentEqual(
@@ -519,22 +543,15 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
             task_name="noop",
             task_data={},
         )
-        root = WorkRequest.objects.create_workflow(
-            template=template,
-            data={},
-            created_by=self.playground.get_default_user(),
-        )
+        root = self.playground.create_workflow(template, task_data={})
         child_template = WorkflowTemplate.objects.create(
             name="child",
             workspace=self.scenario.workspace,
             task_name="noop",
             task_data={},
         )
-        child = WorkRequest.objects.create_workflow(
-            template=child_template,
-            data={},
-            created_by=self.playground.get_default_user(),
-            parent=root,
+        child = self.playground.create_workflow(
+            child_template, task_data={}, parent=root
         )
         grandchildren = []
         for i in range(2):
@@ -611,22 +628,15 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
             task_name="noop",
             task_data={},
         )
-        root = WorkRequest.objects.create_workflow(
-            template=template,
-            data={},
-            created_by=self.playground.get_default_user(),
-        )
+        root = self.playground.create_workflow(template, task_data={})
         child_template = WorkflowTemplate.objects.create(
             name="child",
             workspace=self.scenario.workspace,
             task_name="noop",
             task_data={},
         )
-        child = WorkRequest.objects.create_workflow(
-            template=child_template,
-            data={},
-            created_by=self.playground.get_default_user(),
-            parent=root,
+        child = self.playground.create_workflow(
+            child_template, task_data={}, parent=root
         )
         grandchildren = []
         for i in range(2):
@@ -694,11 +704,8 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
             user, WorkRequest, "change_workrequest"
         )
         template = self.playground.create_workflow_template("test", "noop")
-        root = WorkRequest.objects.create_workflow(
-            template=template,
-            data={},
-            created_by=self.playground.get_default_user(),
-            status=WorkRequest.Statuses.RUNNING,
+        root = self.playground.create_workflow(
+            template, task_data={}, status=WorkRequest.Statuses.RUNNING
         )
         work_request = WorkRequest.objects.create_synchronization_point(
             parent=root, step="test", status=WorkRequest.Statuses.BLOCKED
@@ -721,10 +728,7 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
 
         self.client.force_login(user)
         response = self.client.get(work_request.get_absolute_url())
-
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
-
+        tree = self.assertResponseHTML(response)
         tbody = tree.xpath("//table[@id='review-log']")[0].tbody[0]
         self.assertEqual(len(tbody.tr), 1)
         self.assertEqual(tbody.tr.td[1], self.scenario.user.username)
@@ -761,9 +765,7 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
 
         self.client.force_login(user)
         response = self.client.get(work_request.get_absolute_url())
-
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         requires_signature = tree.xpath("//div[@id='requires-signature']")[0]
         self.assertTextContentEqual(
             requires_signature.div[0], "Waiting for signature"
@@ -790,9 +792,7 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
 
         self.client.force_login(other_user)
         response = self.client.get(work_request.get_absolute_url())
-
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertEqual(tree.xpath("//div[@id='requires-signature']"), [])
 
 
@@ -834,8 +834,7 @@ class WorkRequestListViewTests(ViewTestMixin, TestCase):
                 kwargs={"wname": self.scenario.workspace.name},
             )
         )
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertNoWorkRequests(tree)
 
     def test_get_work_requests_workspace_filter(self) -> None:
@@ -860,8 +859,7 @@ class WorkRequestListViewTests(ViewTestMixin, TestCase):
                 kwargs={"wname": self.scenario.workspace.name},
             )
         )
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertListsWorkRequests(tree, [public_work_request])
 
         response = self.client.get(
@@ -870,13 +868,9 @@ class WorkRequestListViewTests(ViewTestMixin, TestCase):
                 kwargs={"wname": private_workspace.name},
             )
         )
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertListsWorkRequests(tree, [private_work_request])
 
-        # the view have the handle to sort the table
-        self.assertContains(response, _sort_table_handle, html=True)
-
     def test_get_work_requests_exclude_internal(self) -> None:
         """The list excludes INTERNAL work requests."""
         template = WorkflowTemplate.objects.create(
@@ -885,11 +879,7 @@ class WorkRequestListViewTests(ViewTestMixin, TestCase):
             task_data={},
             workspace=self.scenario.workspace,
         )
-        root = WorkRequest.objects.create_workflow(
-            template=template,
-            data={},
-            created_by=self.scenario.user,
-        )
+        root = self.playground.create_workflow(template, task_data={})
         WorkRequest.objects.create_synchronization_point(
             parent=root, step="test"
         )
@@ -901,8 +891,7 @@ class WorkRequestListViewTests(ViewTestMixin, TestCase):
             )
         )
         self.assertEqual(response.status_code, status.HTTP_200_OK)
-
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         assert isinstance(response, SimpleTemplateResponse)
         self.assertQuerySetEqual(response.context_data["object_list"], [root])
 
@@ -914,57 +903,17 @@ class WorkRequestListViewTests(ViewTestMixin, TestCase):
         """Pagination is set up and rendered by the template."""
         self.playground.create_work_request(task_name="noop")
 
-        self.assertGreaterEqual(WorkRequestListView.paginate_by, 10)
         response = self.client.get(
             reverse(
                 "workspaces:work-requests:list",
                 kwargs={"wname": self.scenario.workspace.name},
             )
         )
-        self.assertContains(response, '<nav aria-label="pagination">')
-        self.assertIsInstance(
-            response.context["elided_page_range"], GeneratorType
-        )
-
-    def test_sorting(self) -> None:
-        """Test sorting."""
-        for field in ["id", "created_at", "task_name", "status", "result"]:
-            for asc in ["0", "1"]:
-                response = self.client.get(
-                    reverse(
-                        "workspaces:work-requests:list",
-                        kwargs={"wname": self.scenario.workspace.name},
-                    )
-                    + f"?order={field}&asc={asc}"
-                )
-
-            self.assertEqual(response.context["order"], field)
-            self.assertEqual(response.context["asc"], asc)
-
-    def test_sorting_invalid_field(self) -> None:
-        """Test sorting by a non-valid field: sorted by id."""
-        response = self.client.get(
-            reverse(
-                "workspaces:work-requests:list",
-                kwargs={"wname": self.scenario.workspace.name},
-            )
-            + "?order=something"
-        )
-
-        self.assertEqual(response.context["order"], "created_at")
-        self.assertEqual(response.context["asc"], "0")
-
-    def test_sorting_field_is_valid(self) -> None:
-        """Test sorting with asc=0: valid order and asc."""
-        response = self.client.get(
-            reverse(
-                "workspaces:work-requests:list",
-                kwargs={"wname": self.scenario.workspace.name},
-            )
-            + "?order=id&asc=0"
-        )
-        self.assertEqual(response.context["order"], "id")
-        self.assertEqual(response.context["asc"], "0")
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        tree = self.assertHTMLValid(response)
+        self.assertHasElement(tree, "//table[@id='work_request-list']")
+        pagination = response.context["paginator"]
+        self.assertEqual(pagination.per_page, 50)
 
     def test_filter_by_status(self) -> None:
         """Test get_queryset() with status=XX returns expected WorkRequests."""
@@ -1037,7 +986,8 @@ class WorkRequestListViewTests(ViewTestMixin, TestCase):
 
         assert isinstance(response, SimpleTemplateResponse)
         self.assertQuerySetEqual(
-            response.context_data["work_request_list"], [work_request]
+            response.context_data["paginator"].page_obj.object_list,
+            [work_request],
         )
 
     def test_get_queryset_filter_by_architecture_wrong_status(self) -> None:
@@ -1134,7 +1084,7 @@ class WorkRequestListViewTests(ViewTestMixin, TestCase):
 
         assert isinstance(response, SimpleTemplateResponse)
         self.assertQuerySetEqual(
-            response.context_data["work_request_list"],
+            response.context_data["paginator"].page_obj.object_list,
             [work_request_amd64_2, work_request_amd64_1],
         )
 
@@ -1382,10 +1332,9 @@ class WorkRequestUnblockViewTests(ViewTestMixin, TestCase):
         """Initialize class data."""
         super().setUpTestData()
         template = cls.playground.create_workflow_template("test", "noop")
-        cls.workflow = WorkRequest.objects.create_workflow(
-            template=template,
-            data={},
-            created_by=cls.scenario.user,
+        cls.workflow = cls.playground.create_workflow(
+            template,
+            task_data={},
             status=WorkRequest.Statuses.RUNNING,
         )
         cls.work_request = WorkRequest.objects.create_synchronization_point(
