@@ -43,7 +43,9 @@ from debusine.web.views.base import (
 )
 from debusine.web.views.base_rightbar import RightbarUIView
 from debusine.web.views.http_errors import HttpError400, catch_http_errors
-from debusine.web.views.view_utils import PaginationMixin, format_yaml
+from debusine.web.views.table import TableMixin
+from debusine.web.views.tables import WorkRequestTable
+from debusine.web.views.view_utils import format_yaml
 from debusine.web.web_utils import ui_prototype
 
 
@@ -70,7 +72,7 @@ class WorkRequestObjectMixin(WorkspaceView):
 class WorkRequestDetailView(
     WorkRequestObjectMixin, RightbarUIView, DetailViewBase[WorkRequest]
 ):
-    """List work requests."""
+    """Show a work request."""
 
     model = WorkRequest
     context_object_name = "work_request"
@@ -278,13 +280,15 @@ class WorkRequestPlugin:
 
 
 class WorkRequestListView(
-    WorkRequestObjectMixin, PaginationMixin, ListViewBase[WorkRequest]
+    TableMixin[WorkRequest], WorkRequestObjectMixin, ListViewBase[WorkRequest]
 ):
     """List work requests."""
 
     model = WorkRequest
+    table_class = WorkRequestTable
     template_name = "web/work_request-list.html"
     context_object_name = "work_request_list"
+    title = "List of work requests"
     paginate_by = 50
 
     @staticmethod
@@ -373,26 +377,6 @@ class WorkRequestListView(
 
         return queryset
 
-    def get_ordering(self) -> str:
-        """Return field used for sorting."""
-        order = self.request.GET.get("order")
-        if order in ("id", "created_at", "task_name", "status", "result"):
-            if self.request.GET["asc"] == "0":
-                return "-" + order
-            else:
-                return order
-
-        return "-created_at"
-
-    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
-        """Add context to the default ListView data."""
-        context = super().get_context_data(**kwargs)
-
-        context["order"] = self.get_ordering().removeprefix("-")
-        context["asc"] = self.request.GET.get("asc", "0")
-
-        return context
-
 
 class WorkRequestCreateView(
     WorkRequestObjectMixin,
@@ -403,6 +387,7 @@ class WorkRequestCreateView(
     model = WorkRequest
     template_name = "web/work_request-create.html"
     form_class = WorkRequestForm
+    title = "Create work request"
 
     def init_view(self) -> None:
         """Set the current workspace."""
